import unittest
from unittest.mock import MagicMock, patch

from flask import Flask
from mlflow.exceptions import MlflowException
from mlflow.protos.databricks_pb2 import BAD_REQUEST, INVALID_PARAMETER_VALUE, RESOURCE_DOES_NOT_EXIST

from mlflow_oidc_auth.permissions import Permission
from mlflow_oidc_auth.utils import (
    PermissionResult,
    can_manage_experiment,
    can_manage_registered_model,
    can_read_experiment,
    can_read_registered_model,
    check_experiment_permission,
    check_prompt_permission,
    check_registered_model_permission,
    effective_experiment_permission,
    effective_prompt_permission,
    effective_registered_model_permission,
    get_experiment_id,
    get_is_admin,
    get_optional_request_param,
    get_permission_from_store_or_default,
    get_request_param,
    get_username,
)


class TestUtils(unittest.TestCase):
    def setUp(self):
        self.app = Flask(__name__)
        self.app.config["TESTING"] = True
        self.app_context = self.app.app_context()
        self.app_context.push()
        self.client = self.app.test_client()

    def tearDown(self):
        self.app_context.pop()

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.get_username")
    def test_get_is_admin(self, mock_get_username, mock_store):
        with self.app.test_request_context():
            mock_get_username.return_value = "user"
            mock_store.get_user.return_value.is_admin = True
            self.assertTrue(get_is_admin())
            mock_store.get_user.return_value.is_admin = False
            self.assertFalse(get_is_admin())

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.config")
    @patch("mlflow_oidc_auth.utils.get_permission")
    def test_get_permission_from_store_or_default(self, mock_get_permission, mock_config, mock_store):
        with self.app.test_request_context():
            mock_store_permission_user_func = MagicMock()
            mock_store_permission_group_func = MagicMock()
            mock_store_permission_user_func.return_value = "user_perm"
            mock_store_permission_group_func.return_value = "group_perm"
            mock_get_permission.return_value = Permission(
                name="perm", priority=1, can_read=True, can_update=True, can_delete=True, can_manage=True
            )
            mock_config.PERMISSION_SOURCE_ORDER = ["user", "group"]
            mock_config.DEFAULT_MLFLOW_PERMISSION = "default_perm"

            # user permission found
            result = get_permission_from_store_or_default(
                {"user": mock_store_permission_user_func, "group": mock_store_permission_group_func}
            )
            self.assertTrue(result.permission.can_manage)
            self.assertEqual(result.type, "user")

            # user not found, group found
            mock_store_permission_user_func.side_effect = MlflowException("", RESOURCE_DOES_NOT_EXIST)
            result = get_permission_from_store_or_default(
                {"user": mock_store_permission_user_func, "group": mock_store_permission_group_func}
            )
            self.assertTrue(result.permission.can_manage)
            self.assertEqual(result.type, "group")

            # both not found, fallback to default
            mock_store_permission_group_func.side_effect = MlflowException("", RESOURCE_DOES_NOT_EXIST)
            result = get_permission_from_store_or_default(
                {"user": mock_store_permission_user_func, "group": mock_store_permission_group_func}
            )
            self.assertTrue(result.permission.can_manage)
            self.assertEqual(result.type, "fallback")

            # invalid source in config
            mock_config.PERMISSION_SOURCE_ORDER = ["invalid"]
            # Just call and check fallback, don't assert logs
            result = get_permission_from_store_or_default(
                {"user": mock_store_permission_user_func, "group": mock_store_permission_group_func}
            )
            self.assertEqual(result.type, "fallback")

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.get_permission_from_store_or_default")
    def test_can_manage_experiment(self, mock_get_permission_from_store_or_default, mock_store):
        with self.app.test_request_context():
            mock_get_permission_from_store_or_default.return_value = PermissionResult(
                Permission(name="perm", priority=1, can_read=True, can_update=True, can_delete=True, can_manage=True), "user"
            )
            self.assertTrue(can_manage_experiment("exp_id", "user"))
            mock_get_permission_from_store_or_default.return_value = PermissionResult(
                Permission(name="perm", priority=1, can_read=True, can_update=True, can_delete=True, can_manage=False), "user"
            )
            self.assertFalse(can_manage_experiment("exp_id", "user"))

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.get_permission_from_store_or_default")
    def test_can_manage_registered_model(self, mock_get_permission_from_store_or_default, mock_store):
        with self.app.test_request_context():
            mock_get_permission_from_store_or_default.return_value = PermissionResult(
                Permission(name="perm", priority=1, can_read=True, can_update=True, can_delete=True, can_manage=True), "user"
            )
            self.assertTrue(can_manage_registered_model("model_name", "user"))
            mock_get_permission_from_store_or_default.return_value = PermissionResult(
                Permission(name="perm", priority=1, can_read=True, can_update=True, can_delete=True, can_manage=False), "user"
            )
            self.assertFalse(can_manage_registered_model("model_name", "user"))

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.get_is_admin")
    @patch("mlflow_oidc_auth.utils.get_username")
    @patch("mlflow_oidc_auth.utils.get_experiment_id")
    @patch("mlflow_oidc_auth.utils.can_manage_experiment")
    @patch("mlflow_oidc_auth.utils.make_forbidden_response")
    def test_check_experiment_permission(
        self,
        mock_make_forbidden_response,
        mock_can_manage_experiment,
        mock_get_experiment_id,
        mock_get_username,
        mock_get_is_admin,
        mock_store,
    ):
        with self.app.test_request_context():
            mock_get_is_admin.return_value = False
            mock_get_username.return_value = "user"
            mock_get_experiment_id.return_value = "exp_id"
            mock_can_manage_experiment.return_value = False
            mock_make_forbidden_response.return_value = "forbidden"

            @check_experiment_permission
            def mock_func():
                return "success"

            self.assertEqual(mock_func(), "forbidden")

            mock_can_manage_experiment.return_value = True
            self.assertEqual(mock_func(), "success")

            # Admin always allowed
            mock_get_is_admin.return_value = True
            self.assertEqual(mock_func(), "success")

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.get_is_admin")
    @patch("mlflow_oidc_auth.utils.get_username")
    @patch("mlflow_oidc_auth.utils.get_request_param")
    @patch("mlflow_oidc_auth.utils.can_manage_registered_model")
    @patch("mlflow_oidc_auth.utils.make_forbidden_response")
    def test_check_registered_model_permission(
        self,
        mock_make_forbidden_response,
        mock_can_manage_registered_model,
        mock_get_request_param,
        mock_get_username,
        mock_get_is_admin,
        mock_store,
    ):
        with self.app.test_request_context():
            mock_get_is_admin.return_value = False
            mock_get_username.return_value = "user"
            mock_get_request_param.return_value = "model_name"
            mock_can_manage_registered_model.return_value = False
            mock_make_forbidden_response.return_value = "forbidden"

            @check_registered_model_permission
            def mock_func():
                return "success"

            self.assertEqual(mock_func(), "forbidden")

            mock_can_manage_registered_model.return_value = True
            self.assertEqual(mock_func(), "success")

            # Admin always allowed
            mock_get_is_admin.return_value = True
            self.assertEqual(mock_func(), "success")

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.get_is_admin")
    @patch("mlflow_oidc_auth.utils.get_username")
    @patch("mlflow_oidc_auth.utils.get_request_param")
    @patch("mlflow_oidc_auth.utils.can_manage_registered_model")
    @patch("mlflow_oidc_auth.utils.make_forbidden_response")
    def test_check_prompt_permission(
        self,
        mock_make_forbidden_response,
        mock_can_manage_registered_model,
        mock_get_request_param,
        mock_get_username,
        mock_get_is_admin,
        mock_store,
    ):
        with self.app.test_request_context():
            mock_get_is_admin.return_value = False
            mock_get_username.return_value = "user"
            mock_get_request_param.return_value = "prompt_name"
            mock_can_manage_registered_model.return_value = False
            mock_make_forbidden_response.return_value = "forbidden"

            @check_prompt_permission
            def mock_func():
                return "success"

            self.assertEqual(mock_func(), "forbidden")

            mock_can_manage_registered_model.return_value = True
            self.assertEqual(mock_func(), "success")

            # Admin always allowed
            mock_get_is_admin.return_value = True
            self.assertEqual(mock_func(), "success")

    def test_get_request_param(self):
        # GET method, param present
        with self.app.test_request_context("/?foo=bar", method="GET"):
            self.assertEqual(get_request_param("foo"), "bar")
        # POST method, param present
        with self.app.test_request_context("/", method="POST", json={"foo": "baz"}):
            self.assertEqual(get_request_param("foo"), "baz")
        # param missing, run_id fallback to run_uuid
        with self.app.test_request_context("/", method="GET"):
            with patch("mlflow_oidc_auth.utils.get_request_param", return_value="uuid_val") as mock_get:
                self.assertEqual(get_request_param("run_id"), "uuid_val")
        # param missing, not run_id
        with self.app.test_request_context("/", method="GET"):
            with self.assertRaises(MlflowException) as cm:
                get_request_param("notfound")
            self.assertEqual(cm.exception.error_code, "INVALID_PARAMETER_VALUE")
        # unsupported method
        with self.app.test_request_context("/", method="PUT"):
            with self.assertRaises(MlflowException) as cm:
                get_request_param("foo")
            self.assertEqual(cm.exception.error_code, "BAD_REQUEST")

    def test_get_optional_request_param(self):
        # GET method, param present
        with self.app.test_request_context("/?foo=bar", method="GET"):
            self.assertEqual(get_optional_request_param("foo"), "bar")
        # POST method, param present
        with self.app.test_request_context("/", method="POST", json={"foo": "baz"}):
            self.assertEqual(get_optional_request_param("foo"), "baz")
        # param missing
        with self.app.test_request_context("/", method="GET"):
            self.assertIsNone(get_optional_request_param("notfound"))
        # unsupported method
        with self.app.test_request_context("/", method="PUT"):
            with self.assertRaises(MlflowException) as cm:
                get_optional_request_param("foo")
            self.assertEqual(cm.exception.error_code, "BAD_REQUEST")

    @patch("mlflow_oidc_auth.utils._get_tracking_store")
    def test_get_experiment_id(self, mock_tracking_store):
        # GET method, experiment_id present
        with self.app.test_request_context("/?experiment_id=123", method="GET"):
            self.assertEqual(get_experiment_id(), "123")
        # POST method, experiment_id present
        with self.app.test_request_context("/", method="POST", json={"experiment_id": "456"}):
            self.assertEqual(get_experiment_id(), "456")
        # experiment_name present
        with self.app.test_request_context("/?experiment_name=exp", method="GET"):
            mock_tracking_store().get_experiment_by_name.return_value.experiment_id = "789"
            self.assertEqual(get_experiment_id(), "789")
        # missing both
        with self.app.test_request_context("/", method="GET"):
            with self.assertRaises(MlflowException) as cm:
                get_experiment_id()
            self.assertEqual(cm.exception.error_code, "INVALID_PARAMETER_VALUE")
        # unsupported method
        with self.app.test_request_context("/", method="PUT"):
            with self.assertRaises(MlflowException) as cm:
                get_experiment_id()
            self.assertEqual(cm.exception.error_code, "BAD_REQUEST")

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.validate_token")
    def test_get_username(self, mock_validate_token, mock_store):
        with self.app.test_request_context():
            # session username
            with patch("mlflow_oidc_auth.utils.session", {"username": "session_user"}):
                with patch("mlflow_oidc_auth.utils.request") as mock_request:
                    mock_request.authorization = None
                    self.assertEqual(get_username(), "session_user")
            # basic auth username
            with patch("mlflow_oidc_auth.utils.session", {}):

                class AuthBasic:
                    type = "basic"
                    username = "basic_user"

                with patch("mlflow_oidc_auth.utils.request") as mock_request:
                    mock_request.authorization = AuthBasic()
                    self.assertEqual(get_username(), "basic_user")

                # missing username in basic auth
                class AuthBasicNone:
                    type = "basic"
                    username = None

                with patch("mlflow_oidc_auth.utils.request") as mock_request:
                    mock_request.authorization = AuthBasicNone()
                    with self.assertRaises(MlflowException):
                        get_username()
            # bearer token
            with patch("mlflow_oidc_auth.utils.session", {}):

                class AuthBearer:
                    type = "bearer"
                    token = "tok"

                mock_validate_token.return_value = {"email": "bearer_user"}
                with patch("mlflow_oidc_auth.utils.request") as mock_request:
                    mock_request.authorization = AuthBearer()
                    self.assertEqual(get_username(), "bearer_user")
            # no auth
            with patch("mlflow_oidc_auth.utils.session", {}):
                with patch("mlflow_oidc_auth.utils.request") as mock_request:
                    mock_request.authorization = None
                    with self.assertRaises(MlflowException):
                        get_username()

    @patch("mlflow_oidc_auth.utils._get_tracking_store")
    def test_get_experiment_id_experiment_name_not_found(self, mock_tracking_store):
        # experiment_name provided but not found
        with self.app.test_request_context("/?experiment_name=nonexistent_exp", method="GET"):
            mock_tracking_store().get_experiment_by_name.return_value = None
            with self.assertRaises(MlflowException) as cm:
                get_experiment_id()
            self.assertEqual(cm.exception.error_code, "INVALID_PARAMETER_VALUE")

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.get_permission_from_store_or_default")
    def test_effective_experiment_permission(self, mock_get_permission_from_store_or_default, mock_store):
        with self.app.test_request_context():
            mock_get_permission_from_store_or_default.return_value = PermissionResult(
                Permission(name="perm", priority=1, can_read=True, can_update=True, can_delete=True, can_manage=True), "user"
            )
            result = effective_experiment_permission("exp_id", "user")
            self.assertTrue(result.permission.can_manage)
            self.assertEqual(result.type, "user")

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.get_permission_from_store_or_default")
    def test_effective_registered_model_permission(self, mock_get_permission_from_store_or_default, mock_store):
        with self.app.test_request_context():
            mock_get_permission_from_store_or_default.return_value = PermissionResult(
                Permission(name="perm", priority=1, can_read=True, can_update=True, can_delete=True, can_manage=True), "user"
            )
            result = effective_registered_model_permission("model_name", "user")
            self.assertTrue(result.permission.can_manage)
            self.assertEqual(result.type, "user")

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.get_permission_from_store_or_default")
    def test_effective_prompt_permission(self, mock_get_permission_from_store_or_default, mock_store):
        with self.app.test_request_context():
            mock_get_permission_from_store_or_default.return_value = PermissionResult(
                Permission(name="perm", priority=1, can_read=True, can_update=True, can_delete=True, can_manage=True), "user"
            )
            result = effective_prompt_permission("prompt_name", "user")
            self.assertTrue(result.permission.can_manage)
            self.assertEqual(result.type, "user")

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.get_permission_from_store_or_default")
    def test_can_read_experiment(self, mock_get_permission_from_store_or_default, mock_store):
        with self.app.test_request_context():
            mock_get_permission_from_store_or_default.return_value = PermissionResult(
                Permission(name="perm", priority=1, can_read=True, can_update=False, can_delete=False, can_manage=False), "user"
            )
            self.assertTrue(can_read_experiment("exp_id", "user"))

    @patch("mlflow_oidc_auth.utils.store")
    @patch("mlflow_oidc_auth.utils.get_permission_from_store_or_default")
    def test_can_read_registered_model(self, mock_get_permission_from_store_or_default, mock_store):
        with self.app.test_request_context():
            mock_get_permission_from_store_or_default.return_value = PermissionResult(
                Permission(name="perm", priority=1, can_read=True, can_update=False, can_delete=False, can_manage=False), "user"
            )
            self.assertTrue(can_read_registered_model("model_name", "user"))


if __name__ == "__main__":
    unittest.main()



import pytest
from unittest.mock import MagicMock, patch
from mlflow_oidc_auth.repository.registered_model_permission_regex_group import RegisteredModelGroupRegexPermissionRepository
from mlflow.exceptions import MlflowException


@pytest.fixture
def session():
    s = MagicMock()
    s.__enter__.return_value = s
    s.__exit__.return_value = None
    return s


@pytest.fixture
def session_maker(session):
    return MagicMock(return_value=session)


@pytest.fixture
def repo(session_maker):
    return RegisteredModelGroupRegexPermissionRepository(session_maker)


def test_grant(repo, session):
    group = MagicMock(id=1)
    session.add = MagicMock()
    session.flush = MagicMock()
    with patch("mlflow_oidc_auth.repository.utils.get_group", return_value=group), patch(
        "mlflow_oidc_auth.db.models.SqlRegisteredModelGroupRegexPermission", return_value=MagicMock()
    ):
        result = repo.grant("group1", "r", "READ", priority=1, prompt=True)
        session.add.assert_called_once()
        session.flush.assert_called_once()
        assert result is not None


def test_get(repo, session):
    group = MagicMock(id=2)
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    session.query().filter().one_or_none.return_value = perm
    with patch("mlflow_oidc_auth.repository.utils.get_group", return_value=group):
        result = repo.get("r", "group1", prompt=True)
        assert result == "entity"


def test_get_not_found(repo, session):
    group = MagicMock(id=3)
    session.query().filter().one_or_none.return_value = None
    with patch("mlflow_oidc_auth.repository.utils.get_group", return_value=group):
        with pytest.raises(MlflowException):
            repo.get("r", "group1", prompt=True)


def test_list_permissions_for_group(repo, session):
    group = MagicMock(id=4)
    perm1 = MagicMock()
    perm1.to_mlflow_entity.return_value = "entity1"
    perm2 = MagicMock()
    perm2.to_mlflow_entity.return_value = "entity2"
    session.query().filter().order_by().all.return_value = [perm1, perm2]
    with patch("mlflow_oidc_auth.repository.utils.get_group", return_value=group):
        result = repo.list_permissions_for_group("group1", prompt=True)
        assert result == ["entity1", "entity2"]


def test_list_permissions_for_groups(repo, session):
    group1 = MagicMock(id=5)
    group2 = MagicMock(id=6)
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    session.query().filter().order_by().all.return_value = [perm]
    with patch("mlflow_oidc_auth.repository.utils.get_group", side_effect=[group1, group2]):
        result = repo.list_permissions_for_groups(["group1", "group2"], prompt=True)
        assert result == ["entity"]


def test_list_permissions_for_groups_ids(repo, session):
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    session.query().filter().order_by().all.return_value = [perm]
    result = repo.list_permissions_for_groups_ids([1, 2], prompt=True)
    assert result == ["entity"]


def test_update(repo, session):
    group = MagicMock(id=7)
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    session.query().filter().one_or_none.return_value = perm
    session.flush = MagicMock()
    with patch("mlflow_oidc_auth.repository.utils.get_group", return_value=group):
        result = repo.update("r", "group1", "EDIT", priority=2, prompt=True)
        assert result == "entity"
        assert perm.permission == "EDIT"
        assert perm.priority == 2
        session.flush.assert_called_once()


def test_update_not_found(repo, session):
    group = MagicMock(id=8)
    session.query().filter().one_or_none.return_value = None
    with patch("mlflow_oidc_auth.repository.utils.get_group", return_value=group):
        with pytest.raises(MlflowException):
            repo.update("r", "group1", "EDIT", priority=2, prompt=True)


def test_revoke(repo, session):
    group = MagicMock(id=9)
    perm = MagicMock()
    session.query().filter().one_or_none.return_value = perm
    session.delete = MagicMock()
    session.flush = MagicMock()
    with patch("mlflow_oidc_auth.repository.utils.get_group", return_value=group):
        repo.revoke("r", "group1", prompt=True)
        session.delete.assert_called_once_with(perm)
        session.flush.assert_called_once()


def test_revoke_not_found(repo, session):
    group = MagicMock(id=10)
    session.query().filter().one_or_none.return_value = None
    with patch("mlflow_oidc_auth.repository.utils.get_group", return_value=group):
        with pytest.raises(MlflowException):
            repo.revoke("r", "group1", prompt=True)



from typing import Optional

import requests
from authlib.integrations.flask_client import OAuth
from authlib.jose import jwt
from flask import request
from mlflow.server import app

from mlflow_oidc_auth.config import config
from mlflow_oidc_auth.store import store

_oauth_instance: Optional[OAuth] = None


def get_oauth_instance(app) -> OAuth:
    # returns a singleton instance of OAuth
    # to avoid circular imports
    global _oauth_instance

    if _oauth_instance is None:
        _oauth_instance = OAuth(app)
        _oauth_instance.register(
            name="oidc",
            client_id=config.OIDC_CLIENT_ID,
            client_secret=config.OIDC_CLIENT_SECRET,
            server_metadata_url=config.OIDC_DISCOVERY_URL,
            client_kwargs={"scope": config.OIDC_SCOPE},
        )
    return _oauth_instance


def _get_oidc_jwks():
    from mlflow_oidc_auth.app import cache

    jwks = cache.get("jwks")
    if jwks:
        app.logger.debug("JWKS cache hit")
        return jwks
    app.logger.debug("JWKS cache miss")
    if config.OIDC_DISCOVERY_URL is None:
        raise ValueError("OIDC_DISCOVERY_URL is not set in the configuration")
    metadata = requests.get(config.OIDC_DISCOVERY_URL).json()
    jwks_uri = metadata.get("jwks_uri")
    jwks = requests.get(jwks_uri).json()
    cache.set("jwks", jwks, timeout=3600)
    return jwks


def validate_token(token):
    jwks = _get_oidc_jwks()
    payload = jwt.decode(token, jwks)
    payload.validate()
    return payload


def authenticate_request_basic_auth() -> bool:
    if request.authorization is None:
        return False
    username = request.authorization.username
    password = request.authorization.password
    app.logger.debug("Authenticating user %s", username)
    if username is not None and password is not None and store.authenticate_user(username.lower(), password):
        app.logger.debug("User %s authenticated", username)
        return True
    else:
        app.logger.debug("User %s not authenticated", username)
        return False


def authenticate_request_bearer_token() -> bool:
    if request.authorization and request.authorization.token:
        token = request.authorization.token
        try:
            user = validate_token(token)
            app.logger.debug("User %s authenticated", user.get("email"))
            return True
        except Exception as e:
            app.logger.debug("JWT auth failed")
            return False
    else:
        app.logger.debug("No authorization token found")
        return False



import unittest
from unittest.mock import Mock, patch
from mlflow_oidc_auth.plugins.group_detection_microsoft_entra_id import get_user_groups


class TestGetUserGroups(unittest.TestCase):
    @patch("mlflow_oidc_auth.plugins.group_detection_microsoft_entra_id.requests.get")
    def test_get_user_groups(self, mock_get):
        mock_response = Mock()
        mock_response.json.return_value = {
            "value": [
                {"displayName": "Group 1"},
                {"displayName": "Group 2"},
                {"displayName": "Group 3"},
                {"displayName": "Group 3"},
                {"displayName": None},
            ]
        }
        mock_get.return_value = mock_response

        access_token = "D34DB33F"
        groups = get_user_groups(access_token)

        mock_get.assert_called_once_with(
            "https://graph.microsoft.com/v1.0/me/memberOf",
            headers={
                "Authorization": f"Bearer {access_token}",
                "Content-Type": "application/json",
            },
        )

        expected_groups = ["Group 1", "Group 2", "Group 3"]
        self.assertEqual(groups, expected_groups)



import pytest
from unittest.mock import MagicMock, patch
from mlflow_oidc_auth.repository.prompt_permission_group import PromptPermissionGroupRepository


@pytest.fixture
def session():
    s = MagicMock()
    s.__enter__.return_value = s
    s.__exit__.return_value = None
    return s


@pytest.fixture
def session_maker(session):
    return MagicMock(return_value=session)


@pytest.fixture
def repo(session_maker):
    return PromptPermissionGroupRepository(session_maker)


def test_list_prompt_permissions_for_group(repo, session):
    group = MagicMock(id=2)
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    session.query().filter().all.return_value = [perm]
    with patch("mlflow_oidc_auth.repository.prompt_permission_group.get_group", return_value=group):
        result = repo.list_prompt_permissions_for_group("g")
        assert result == ["entity"]


def test_update_prompt_permission_for_group(repo, session):
    group = MagicMock(id=3)
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    session.query().filter().one.return_value = perm
    session.flush = MagicMock()
    with patch("mlflow_oidc_auth.repository.prompt_permission_group.get_group", return_value=group):
        result = repo.update_prompt_permission_for_group("g", "prompt", "EDIT")
        assert result == "entity"
        assert perm.permission == "EDIT"
        session.flush.assert_called_once()


def test_revoke_prompt_permission_from_group(repo, session):
    group = MagicMock(id=4)
    perm = MagicMock()
    session.query().filter().one.return_value = perm
    session.delete = MagicMock()
    session.flush = MagicMock()
    with patch("mlflow_oidc_auth.repository.prompt_permission_group.get_group", return_value=group):
        repo.revoke_prompt_permission_from_group("g", "prompt")
        session.delete.assert_called_once_with(perm)
        session.flush.assert_called_once()



from flask import jsonify
from mlflow.server.handlers import catch_mlflow_exception

from mlflow_oidc_auth.store import store
from mlflow_oidc_auth.utils import check_admin_permission, get_request_param


@catch_mlflow_exception
@check_admin_permission
def create_group_registered_model_regex_permission(group_name):
    store.create_group_registered_model_regex_permission(
        group_name=group_name,
        regex=get_request_param("regex"),
        priority=int(get_request_param("priority")),
        permission=get_request_param("permission"),
    )
    return jsonify({"status": "success"}), 200


@catch_mlflow_exception
@check_admin_permission
def get_group_registered_model_regex_permission(group_name):
    ep = store.list_group_registered_model_regex_permissions(
        group_name=group_name,
    )
    return jsonify([e.to_json() for e in ep]), 200


@catch_mlflow_exception
@check_admin_permission
def update_group_registered_model_regex_permission(group_name):
    ep = store.update_group_registered_model_regex_permission(
        group_name=group_name,
        regex=get_request_param("regex"),
        priority=int(get_request_param("priority")),
        permission=get_request_param("permission"),
    )
    return jsonify({"registered_model_permission": ep.to_json()}), 200


@catch_mlflow_exception
@check_admin_permission
def delete_group_registered_model_regex_permission(group_name):
    store.delete_group_registered_model_regex_permission(
        group_name=group_name,
        regex=get_request_param("regex"),
    )
    return jsonify({"status": "success"}), 200



from unittest.mock import patch

from flask import Flask, Request

from mlflow_oidc_auth.validators import user


def test__username_is_sender_true():
    with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value="alice"), patch(
        "mlflow_oidc_auth.validators.user.get_username", return_value="alice"
    ):
        assert user._username_is_sender() is True


def test__username_is_sender_false():
    with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value="alice"), patch(
        "mlflow_oidc_auth.validators.user.get_username", return_value="bob"
    ):
        assert user._username_is_sender() is False


def test__username_is_sender_none_username():
    with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value=None), patch(
        "mlflow_oidc_auth.validators.user.get_username", return_value="bob"
    ):
        assert user._username_is_sender() is False


def test__username_is_sender_none_sender():
    with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value="alice"), patch(
        "mlflow_oidc_auth.validators.user.get_username", return_value=None
    ):
        assert user._username_is_sender() is False


def test__username_is_sender_both_none():
    with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value=None), patch(
        "mlflow_oidc_auth.validators.user.get_username", return_value=None
    ):
        assert user._username_is_sender() is True  # None == None


def test_validate_can_get_user_token():
    app = Flask(__name__)
    with app.test_request_context(method="GET"):
        with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value="alice"), patch(
            "mlflow_oidc_auth.validators.user.get_username", return_value="alice"
        ):
            assert user.validate_can_get_user_token() is True


def test_validate_cant_get_user_token():
    app = Flask(__name__)
    with app.test_request_context(method="GET"):
        with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value="alice"), patch(
            "mlflow_oidc_auth.validators.user.get_username", return_value="bob"
        ):
            assert user.validate_can_get_user_token() is False


def test_validate_can_create_user():
    assert user.validate_can_create_user() is False


def test_validate_can_update_user_admin():
    assert user.validate_can_update_user_admin() is False


def test_validate_can_delete_user():
    assert user.validate_can_delete_user() is False


def test_validate_can_read_user_true():
    with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value="alice"), patch(
        "mlflow_oidc_auth.validators.user.get_username", return_value="alice"
    ):
        assert user.validate_can_read_user() is True


def test_validate_can_read_user_false():
    with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value="alice"), patch(
        "mlflow_oidc_auth.validators.user.get_username", return_value="bob"
    ):
        assert user.validate_can_read_user() is False


def test_validate_can_update_user_password_true():
    with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value="alice"), patch(
        "mlflow_oidc_auth.validators.user.get_username", return_value="alice"
    ):
        assert user.validate_can_update_user_password() is True


def test_validate_can_update_user_password_false():
    with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value="alice"), patch(
        "mlflow_oidc_auth.validators.user.get_username", return_value="bob"
    ):
        assert user.validate_can_update_user_password() is False


def test_validate_can_update_user_password_none():
    with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value=None), patch(
        "mlflow_oidc_auth.validators.user.get_username", return_value="bob"
    ):
        assert user.validate_can_update_user_password() is False


def test_validate_can_update_user_password_both_none():
    with patch("mlflow_oidc_auth.validators.user.get_request_param", return_value=None), patch(
        "mlflow_oidc_auth.validators.user.get_username", return_value=None
    ):
        assert user.validate_can_update_user_password() is True  # None == None



from unittest.mock import MagicMock, patch

import pytest

from mlflow_oidc_auth.sqlalchemy_store import SqlAlchemyStore


@pytest.fixture
@patch("mlflow_oidc_auth.sqlalchemy_store.dbutils.migrate_if_needed")
def store(_mock_migrate_if_needed):
    store = SqlAlchemyStore()
    store.init_db("sqlite:///:memory:")
    return store


class TestSqlAlchemyStore:
    def test_create_experiment_regex_permission(self, store: SqlAlchemyStore):
        store.experiment_regex_repo = MagicMock()
        store.create_experiment_regex_permission(".*", 1, "READ", "user")
        store.experiment_regex_repo.grant.assert_called_once_with(".*", 1, "READ", "user")

    def test_get_experiment_regex_permission(self, store: SqlAlchemyStore):
        store.experiment_regex_repo = MagicMock()
        store.get_experiment_regex_permission(".*", "user")
        store.experiment_regex_repo.get.assert_called_once_with(".*", "user")

    def test_update_experiment_regex_permission(self, store: SqlAlchemyStore):
        store.experiment_regex_repo = MagicMock()
        store.update_experiment_regex_permission(".*", 1, "EDIT", "user")
        store.experiment_regex_repo.update.assert_called_once_with(".*", 1, "EDIT", "user")

    def test_delete_experiment_regex_permission(self, store: SqlAlchemyStore):
        store.experiment_regex_repo = MagicMock()
        store.delete_experiment_regex_permission(".*", "user")
        store.experiment_regex_repo.revoke.assert_called_once_with(".*", "user")

    def test_create_group_experiment_regex_permission(self, store: SqlAlchemyStore):
        store.experiment_group_regex_repo = MagicMock()
        store.create_group_experiment_regex_permission("group", ".*", 1, "READ")
        store.experiment_group_regex_repo.grant.assert_called_once_with("group", ".*", 1, "READ")

    def test_get_group_experiment_regex_permission(self, store: SqlAlchemyStore):
        store.experiment_group_regex_repo = MagicMock()
        store.get_group_experiment_regex_permission("group", ".*")
        store.experiment_group_regex_repo.get.assert_called_once_with("group", ".*")

    def test_list_group_experiment_regex_permissions(self, store: SqlAlchemyStore):
        store.experiment_group_regex_repo = MagicMock()
        store.list_group_experiment_regex_permissions("group")
        store.experiment_group_regex_repo.list_permissions_for_group.assert_called_once_with("group")

    def test_update_group_experiment_regex_permission(self, store: SqlAlchemyStore):
        store.experiment_group_regex_repo = MagicMock()
        store.update_group_experiment_regex_permission("group", ".*", 1, "EDIT")
        store.experiment_group_regex_repo.update.assert_called_once_with("group", ".*", 1, "EDIT")

    def test_delete_group_experiment_regex_permission(self, store: SqlAlchemyStore):
        store.experiment_group_regex_repo = MagicMock()
        store.delete_group_experiment_regex_permission("group", ".*")
        store.experiment_group_regex_repo.revoke.assert_called_once_with("group", ".*")

    def test_create_registered_model_regex_permission(self, store: SqlAlchemyStore):
        store.registered_model_regex_repo = MagicMock()
        store.create_registered_model_regex_permission(".*", 1, "READ", "user")
        store.registered_model_regex_repo.grant.assert_called_once_with(".*", 1, "READ", "user")

    def test_get_registered_model_regex_permission(self, store: SqlAlchemyStore):
        store.registered_model_regex_repo = MagicMock()
        store.get_registered_model_regex_permission(".*", "user")
        store.registered_model_regex_repo.get.assert_called_once_with(".*", "user")

    def test_update_registered_model_regex_permission(self, store: SqlAlchemyStore):
        store.registered_model_regex_repo = MagicMock()
        store.update_registered_model_regex_permission(".*", 1, "EDIT", "user")
        store.registered_model_regex_repo.update.assert_called_once_with(".*", 1, "EDIT", "user")

    def test_delete_registered_model_regex_permission(self, store: SqlAlchemyStore):
        store.registered_model_regex_repo = MagicMock()
        store.delete_registered_model_regex_permission(".*", "user")
        store.registered_model_regex_repo.revoke.assert_called_once_with(".*", "user")

    def test_create_group_registered_model_regex_permission(self, store: SqlAlchemyStore):
        store.registered_model_group_regex_repo = MagicMock()
        store.create_group_registered_model_regex_permission("group", ".*", 1, "READ")
        store.registered_model_group_regex_repo.grant.assert_called_once_with(
            group_name="group", regex=".*", priority=1, permission="READ"
        )

    def test_get_group_registered_model_regex_permission(self, store: SqlAlchemyStore):
        store.registered_model_group_regex_repo = MagicMock()
        store.get_group_registered_model_regex_permission("group", ".*")
        store.registered_model_group_regex_repo.get.assert_called_once_with("group", ".*")

    def test_list_group_registered_model_regex_permissions(self, store: SqlAlchemyStore):
        store.registered_model_group_regex_repo = MagicMock()
        store.list_group_registered_model_regex_permissions("group")
        store.registered_model_group_regex_repo.list_permissions_for_group.assert_called_once_with("group")

    def test_update_group_registered_model_regex_permission(self, store: SqlAlchemyStore):
        store.registered_model_group_regex_repo = MagicMock()
        store.update_group_registered_model_regex_permission("group", ".*", 1, "EDIT")
        store.registered_model_group_regex_repo.update.assert_called_once_with(
            group_name="group", regex=".*", priority=1, permission="EDIT"
        )

    def test_delete_group_registered_model_regex_permission(self, store: SqlAlchemyStore):
        store.registered_model_group_regex_repo = MagicMock()
        store.delete_group_registered_model_regex_permission("group", ".*")
        store.registered_model_group_regex_repo.revoke.assert_called_once_with(group_name="group", regex=".*")

    def test_create_prompt_regex_permission(self, store: SqlAlchemyStore):
        store.prompt_regex_repo = MagicMock()
        store.create_prompt_regex_permission(".*", 1, "READ", "user", prompt=True)
        store.prompt_regex_repo.grant.assert_called_once_with(
            regex=".*", priority=1, permission="READ", username="user", prompt=True
        )

    def test_get_prompt_regex_permission(self, store: SqlAlchemyStore):
        store.prompt_regex_repo = MagicMock()
        store.get_prompt_regex_permission(".*", "user", prompt=True)
        store.prompt_regex_repo.get.assert_called_once_with(regex=".*", username="user", prompt=True)

    def test_update_prompt_regex_permission(self, store: SqlAlchemyStore):
        store.prompt_regex_repo = MagicMock()
        store.update_prompt_regex_permission(".*", 1, "EDIT", "user", prompt=True)
        store.prompt_regex_repo.update.assert_called_once_with(
            regex=".*", priority=1, permission="EDIT", username="user", prompt=True
        )

    def test_delete_prompt_regex_permission(self, store: SqlAlchemyStore):
        store.prompt_regex_repo = MagicMock()
        store.delete_prompt_regex_permission(".*", "user")
        store.prompt_regex_repo.revoke.assert_called_once_with(regex=".*", username="user", prompt=True)

    def test_create_group_prompt_regex_permission(self, store: SqlAlchemyStore):
        store.prompt_group_regex_repo = MagicMock()
        store.create_group_prompt_regex_permission(".*", 1, "READ", "group", prompt=True)
        store.prompt_group_regex_repo.grant.assert_called_once_with(
            regex=".*", priority=1, permission="READ", group_name="group", prompt=True
        )

    def test_get_group_prompt_regex_permission(self, store: SqlAlchemyStore):
        store.prompt_group_regex_repo = MagicMock()
        store.get_group_prompt_regex_permission(".*", "group", prompt=True)
        store.prompt_group_regex_repo.get.assert_called_once_with(regex=".*", group_name="group", prompt=True)

    def test_list_group_prompt_regex_permissions(self, store: SqlAlchemyStore):
        store.prompt_group_regex_repo = MagicMock()
        store.list_group_prompt_regex_permissions("group", prompt=True)
        store.prompt_group_regex_repo.list_permissions_for_group.assert_called_once_with(group_name="group", prompt=True)

    def test_update_group_prompt_regex_permission(self, store: SqlAlchemyStore):
        store.prompt_group_regex_repo = MagicMock()
        store.update_group_prompt_regex_permission(".*", 1, "EDIT", "group", prompt=True)
        store.prompt_group_regex_repo.update.assert_called_once_with(
            regex=".*", priority=1, permission="EDIT", group_name="group", prompt=True
        )

    def test_delete_group_prompt_regex_permission(self, store: SqlAlchemyStore):
        store.prompt_group_regex_repo = MagicMock()
        store.delete_group_prompt_regex_permission(".*", "group")
        store.prompt_group_regex_repo.revoke.assert_called_once_with(regex=".*", group_name="group", prompt=True)

    def test_list_group_prompt_regex_permissions_for_groups(self, store: SqlAlchemyStore):
        store.prompt_group_regex_repo = MagicMock()
        store.list_group_prompt_regex_permissions_for_groups(["group1", "group2"], prompt=True)
        store.prompt_group_regex_repo.list_permissions_for_groups.assert_called_once_with(
            group_names=["group1", "group2"], prompt=True
        )

    def test_list_group_prompt_regex_permissions_for_groups_ids(self, store: SqlAlchemyStore):
        store.prompt_group_regex_repo = MagicMock()
        store.list_group_prompt_regex_permissions_for_groups_ids([1, 2], prompt=True)
        store.prompt_group_regex_repo.list_permissions_for_groups_ids.assert_called_once_with(group_ids=[1, 2], prompt=True)



from typing import List, Callable

from mlflow.exceptions import MlflowException
from mlflow.protos.databricks_pb2 import RESOURCE_ALREADY_EXISTS
from sqlalchemy.exc import IntegrityError

from mlflow_oidc_auth.db.models import SqlExperimentRegexPermission, SqlUser
from mlflow_oidc_auth.entities import ExperimentRegexPermission
from mlflow_oidc_auth.permissions import _validate_permission
from mlflow_oidc_auth.repository.utils import get_one_or_raise, get_user, validate_regex
from sqlalchemy.orm import Session


class ExperimentPermissionRegexRepository:
    def __init__(self, session_maker):
        self._Session: Callable[[], Session] = session_maker

    def grant(
        self,
        regex: str,
        priority: int,
        permission: str,
        username: str,
    ) -> ExperimentRegexPermission:
        validate_regex(regex)
        _validate_permission(permission)
        with self._Session() as session:
            try:
                user = get_user(session, username)
                perm = SqlExperimentRegexPermission(
                    regex=regex,
                    priority=priority,
                    user_id=user.id,
                    permission=permission,
                )
                session.add(perm)
                session.flush()
                return perm.to_mlflow_entity()
            except IntegrityError as e:
                raise MlflowException(
                    f"Experiment perm exists ({regex},{username}): {e}",
                    RESOURCE_ALREADY_EXISTS,
                )

    def get(self, regex: str, username: str) -> ExperimentRegexPermission:
        with self._Session() as session:
            row: SqlExperimentRegexPermission = get_one_or_raise(
                session,
                SqlExperimentRegexPermission,
                SqlExperimentRegexPermission.regex == regex,
                SqlExperimentRegexPermission.user_id == session.query(SqlUser.id).filter(SqlUser.username == username),
                not_found_msg=f"No experiment perm for regex={regex}, user={username}",
                multiple_msg=f"Multiple experiment perms for regex={regex}, user={username}",
            )
            return row.to_mlflow_entity()

    def list(self) -> List[ExperimentRegexPermission]:
        with self._Session() as session:
            rows = session.query(SqlExperimentRegexPermission).all()
            return [r.to_mlflow_entity() for r in rows]

    def list_regex_for_user(self, username: str) -> List[ExperimentRegexPermission]:
        with self._Session() as session:
            user = get_user(session, username)
            rows = session.query(SqlExperimentRegexPermission).filter(SqlExperimentRegexPermission.user_id == user.id).all()
            return [r.to_mlflow_entity() for r in rows]

    def update(self, regex: str, priority: int, permission: str, username: str) -> ExperimentRegexPermission:
        validate_regex(regex)
        _validate_permission(permission)
        with self._Session() as session:
            perm: SqlExperimentRegexPermission = get_one_or_raise(
                session,
                SqlExperimentRegexPermission,
                SqlExperimentRegexPermission.regex == regex,
                SqlExperimentRegexPermission.user_id == session.query(SqlUser.id).filter(SqlUser.username == username),
                not_found_msg=f"No perm to update for regex={regex}, user={username}",
                multiple_msg=f"Multiple perms for regex={regex}, user={username}",
            )
            perm.priority = priority
            perm.permission = permission
            session.flush()
            return perm.to_mlflow_entity()

    def revoke(self, regex: str, username: str) -> None:
        validate_regex(regex)
        with self._Session() as session:
            perm: SqlExperimentRegexPermission = get_one_or_raise(
                session,
                SqlExperimentRegexPermission,
                SqlExperimentRegexPermission.regex == regex,
                SqlExperimentRegexPermission.user_id == session.query(SqlUser.id).filter(SqlUser.username == username),
                not_found_msg=f"No perm to delete for regex={regex}, user={username}",
                multiple_msg=f"Multiple perms for regex={regex}, user={username}",
            )
            session.delete(perm)
            session.commit()
            return None



from flask import jsonify
from mlflow.server.handlers import catch_mlflow_exception

from mlflow_oidc_auth.store import store


@catch_mlflow_exception
def get_groups():
    return jsonify({"groups": store.get_groups()})


@catch_mlflow_exception
def get_group_users(group_name):
    return jsonify({"users": store.get_group_users(group_name)})



import pytest
from unittest.mock import MagicMock, patch
from mlflow_oidc_auth.repository.registered_model_permission import RegisteredModelPermissionRepository
from mlflow.exceptions import MlflowException


@pytest.fixture
def session():
    s = MagicMock()
    s.__enter__.return_value = s
    s.__exit__.return_value = None
    return s


@pytest.fixture
def session_maker(session):
    return MagicMock(return_value=session)


@pytest.fixture
def repo(session_maker):
    return RegisteredModelPermissionRepository(session_maker)


def test_create_integrity_error(repo, session):
    user = MagicMock(id=2)
    session.add = MagicMock()
    session.flush = MagicMock(side_effect=Exception("IntegrityError"))
    with patch("mlflow_oidc_auth.repository.registered_model_permission.get_user", return_value=user), patch(
        "mlflow_oidc_auth.db.models.SqlRegisteredModelPermission", return_value=MagicMock()
    ), patch("mlflow_oidc_auth.repository.registered_model_permission.IntegrityError", Exception):
        with pytest.raises(MlflowException):
            repo.create("name", "user", "READ")


def test_get(repo, session):
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    with patch("mlflow_oidc_auth.repository.registered_model_permission.get_one_or_raise", return_value=perm):
        assert repo.get("name", "user") == "entity"


def test_list_for_user(repo, session):
    user = MagicMock(id=3)
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    session.query().filter().all.return_value = [perm]
    with patch("mlflow_oidc_auth.repository.registered_model_permission.get_user", return_value=user):
        assert repo.list_for_user("user") == ["entity"]


def test_update(repo, session):
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    with patch("mlflow_oidc_auth.repository.registered_model_permission.get_one_or_raise", return_value=perm):
        session.flush = MagicMock()
        result = repo.update("name", "user", "EDIT")
        assert result == "entity"
        assert perm.permission == "EDIT"
        session.flush.assert_called_once()


def test_delete(repo, session):
    perm = MagicMock()
    with patch("mlflow_oidc_auth.repository.registered_model_permission.get_one_or_raise", return_value=perm):
        session.delete = MagicMock()
        session.flush = MagicMock()
        repo.delete("name", "user")
        session.delete.assert_called_once_with(perm)
        session.flush.assert_called_once()


def test_wipe(repo, session):
    perm1 = MagicMock()
    perm2 = MagicMock()
    session.query().filter().all.return_value = [perm1, perm2]
    session.delete = MagicMock()
    session.flush = MagicMock()
    repo.wipe("name")
    assert session.delete.call_count == 2
    session.flush.assert_called_once()



import pytest
from unittest.mock import MagicMock, patch
from mlflow_oidc_auth.repository.registered_model_permission_regex import RegisteredModelPermissionRegexRepository
from mlflow.exceptions import MlflowException


@pytest.fixture
def session():
    s = MagicMock()
    s.__enter__.return_value = s
    s.__exit__.return_value = None
    return s


@pytest.fixture
def session_maker(session):
    return MagicMock(return_value=session)


@pytest.fixture
def repo(session_maker):
    return RegisteredModelPermissionRegexRepository(session_maker)


def test_grant_integrity_error(repo, session):
    user = MagicMock(id=2)
    session.add = MagicMock()
    session.flush = MagicMock(side_effect=Exception("IntegrityError"))
    with patch("mlflow_oidc_auth.repository.registered_model_permission_regex.get_user", return_value=user), patch(
        "mlflow_oidc_auth.db.models.SqlRegisteredModelRegexPermission", return_value=MagicMock()
    ), patch("mlflow_oidc_auth.repository.registered_model_permission_regex.IntegrityError", Exception):
        with pytest.raises(MlflowException):
            repo.grant("r", 1, "READ", "user", prompt=True)


def test_get(repo, session):
    user = MagicMock(id=3)
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    with patch("mlflow_oidc_auth.repository.registered_model_permission_regex.get_user", return_value=user), patch(
        "mlflow_oidc_auth.repository.registered_model_permission_regex.get_one_or_raise", return_value=perm
    ):
        result = repo.get("r", "user", prompt=True)
        assert result == "entity"


def test_list_regex_for_user(repo, session):
    user = MagicMock(id=7)
    perm1 = MagicMock()
    perm1.to_mlflow_entity.return_value = "entity1"
    perm2 = MagicMock()
    perm2.to_mlflow_entity.return_value = "entity2"
    with patch("mlflow_oidc_auth.repository.registered_model_permission_regex.get_user", return_value=user), patch(
        "mlflow_oidc_auth.repository.registered_model_permission_regex.get_all", return_value=[perm1, perm2]
    ):
        result = repo.list_regex_for_user("user", prompt=True)
        assert result == ["entity1", "entity2"]


def test_update(repo, session):
    user = MagicMock(id=5)
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    session.commit = MagicMock()
    with patch("mlflow_oidc_auth.repository.registered_model_permission_regex.get_user", return_value=user), patch(
        "mlflow_oidc_auth.repository.registered_model_permission_regex.get_one_or_raise", return_value=perm
    ):
        result = repo.update("r", 2, "EDIT", "user", prompt=True)
        assert result == "entity"
        assert perm.priority == 2
        assert perm.permission == "EDIT"
        session.commit.assert_called_once()


def test_revoke(repo, session):
    user = MagicMock(id=6)
    perm = MagicMock()
    session.delete = MagicMock()
    session.commit = MagicMock()
    with patch("mlflow_oidc_auth.repository.registered_model_permission_regex.get_user", return_value=user), patch(
        "mlflow_oidc_auth.repository.registered_model_permission_regex.get_one_or_raise", return_value=perm
    ):
        repo.revoke("r", "user", prompt=True)
        session.delete.assert_called_once_with(perm)
        session.commit.assert_called_once()



import os

from flask_caching import Cache
from flask_session import Session
from mlflow.server import app

from mlflow_oidc_auth import routes, views
from mlflow_oidc_auth.config import config
from mlflow_oidc_auth.hooks import after_request_hook, before_request_hook

# Configure custom Flask app
template_dir = os.path.dirname(__file__)
template_dir = os.path.join(template_dir, "templates")

app.config.from_object(config)
app.secret_key = app.config["SECRET_KEY"].encode("utf8")
app.template_folder = template_dir
static_folder = app.static_folder

# Add links to MLFlow UI
app.view_functions["serve"] = views.index

# OIDC routes
app.add_url_rule(rule=routes.LOGIN, methods=["GET"], view_func=views.login)
app.add_url_rule(rule=routes.LOGOUT, methods=["GET"], view_func=views.logout)
app.add_url_rule(rule=routes.CALLBACK, methods=["GET"], view_func=views.callback)

# UI routes
app.add_url_rule(rule=routes.STATIC, methods=["GET"], view_func=views.oidc_static)
app.add_url_rule(rule=routes.UI, methods=["GET"], view_func=views.oidc_ui)
app.add_url_rule(rule=routes.UI_ROOT, methods=["GET"], view_func=views.oidc_ui)

# User token
app.add_url_rule(rule=routes.CREATE_ACCESS_TOKEN, methods=["PATCH"], view_func=views.create_user_access_token)
app.add_url_rule(rule=routes.GET_CURRENT_USER, methods=["GET"], view_func=views.get_current_user)

# UI routes support
app.add_url_rule(rule=routes.GET_EXPERIMENTS, methods=["GET"], view_func=views.get_experiments)
app.add_url_rule(rule=routes.GET_MODELS, methods=["GET"], view_func=views.get_registered_models)
app.add_url_rule(rule=routes.GET_USERS, methods=["GET"], view_func=views.get_users)
app.add_url_rule(rule=routes.GET_USER_EXPERIMENTS, methods=["GET"], view_func=views.get_user_experiments)
app.add_url_rule(rule=routes.GET_USER_MODELS, methods=["GET"], view_func=views.get_user_models)
app.add_url_rule(rule=routes.GET_EXPERIMENT_USERS, methods=["GET"], view_func=views.get_experiment_users)
app.add_url_rule(rule=routes.GET_MODEL_USERS, methods=["GET"], view_func=views.get_registered_model_users)
app.add_url_rule(rule=routes.GET_USER_PROMPTS, methods=["GET"], view_func=views.get_user_prompts)
app.add_url_rule(rule=routes.GET_PROMPT_USERS, methods=["GET"], view_func=views.get_prompt_users)


# User management
app.add_url_rule(rule=routes.CREATE_USER, methods=["POST"], view_func=views.create_new_user)
app.add_url_rule(rule=routes.GET_USER, methods=["GET"], view_func=views.get_user)
app.add_url_rule(rule=routes.UPDATE_USER_PASSWORD, methods=["PATCH"], view_func=views.update_username_password)
app.add_url_rule(rule=routes.UPDATE_USER_ADMIN, methods=["PATCH"], view_func=views.update_user_admin)
app.add_url_rule(rule=routes.DELETE_USER, methods=["DELETE"], view_func=views.delete_user)

# permission management
app.add_url_rule(rule=routes.CREATE_EXPERIMENT_PERMISSION, methods=["POST"], view_func=views.create_experiment_permission)
app.add_url_rule(rule=routes.GET_EXPERIMENT_PERMISSION, methods=["GET"], view_func=views.get_experiment_permission)
app.add_url_rule(rule=routes.UPDATE_EXPERIMENT_PERMISSION, methods=["PATCH"], view_func=views.update_experiment_permission)
app.add_url_rule(rule=routes.DELETE_EXPERIMENT_PERMISSION, methods=["DELETE"], view_func=views.delete_experiment_permission)

# prompt management
app.add_url_rule(rule=routes.CREATE_PROMPT_PERMISSION, methods=["POST"], view_func=views.create_prompt_permission)
app.add_url_rule(rule=routes.GET_PROMPTS, methods=["GET"], view_func=views.get_prompts)
app.add_url_rule(rule=routes.UPDATE_PROMPT_PERMISSION, methods=["PATCH"], view_func=views.update_prompt_permission)
app.add_url_rule(rule=routes.DELETE_PROMPT_PERMISSION, methods=["DELETE"], view_func=views.delete_prompt_permission)

app.add_url_rule(
    rule=routes.CREATE_REGISTERED_MODEL_PERMISSION, methods=["POST"], view_func=views.create_registered_model_permission
)
app.add_url_rule(rule=routes.GET_REGISTERED_MODEL_PERMISSION, methods=["GET"], view_func=views.get_registered_model_permission)
app.add_url_rule(
    rule=routes.UPDATE_REGISTERED_MODEL_PERMISSION, methods=["PATCH"], view_func=views.update_registered_model_permission
)
app.add_url_rule(
    rule=routes.DELETE_REGISTERED_MODEL_PERMISSION, methods=["DELETE"], view_func=views.delete_registered_model_permission
)

app.add_url_rule(rule=routes.GET_GROUPS, methods=["GET"], view_func=views.get_groups)
app.add_url_rule(rule=routes.GET_GROUP_USERS, methods=["GET"], view_func=views.get_group_users)
app.add_url_rule(rule=routes.GET_GROUP_EXPERIMENTS_PERMISSION, methods=["GET"], view_func=views.get_group_experiments)
app.add_url_rule(
    rule=routes.CREATE_GROUP_EXPERIMENT_PERMISSION, methods=["POST"], view_func=views.create_group_experiment_permission
)
app.add_url_rule(
    rule=routes.DELETE_GROUP_EXPERIMENT_PERMISSION, methods=["DELETE"], view_func=views.delete_group_experiment_permission
)
app.add_url_rule(
    rule=routes.UPDATE_GROUP_EXPERIMENT_PERMISSION, methods=["PATCH"], view_func=views.update_group_experiment_permission
)
app.add_url_rule(rule=routes.GET_GROUP_MODELS_PERMISSION, methods=["GET"], view_func=views.get_group_models)
app.add_url_rule(rule=routes.CREATE_GROUP_MODEL_PERMISSION, methods=["POST"], view_func=views.create_group_model_permission)
app.add_url_rule(rule=routes.DELETE_GROUP_MODEL_PERMISSION, methods=["DELETE"], view_func=views.delete_group_model_permission)
app.add_url_rule(rule=routes.UPDATE_GROUP_MODEL_PERMISSION, methods=["PATCH"], view_func=views.update_group_model_permission)

app.add_url_rule(rule=routes.GET_GROUP_PROMPTS_PERMISSION, methods=["GET"], view_func=views.get_group_prompts)
app.add_url_rule(rule=routes.CREATE_GROUP_PROMPT_PERMISSION, methods=["POST"], view_func=views.create_group_prompt_permission)
app.add_url_rule(rule=routes.DELETE_GROUP_PROMPT_PERMISSION, methods=["DELETE"], view_func=views.delete_group_prompt_permission)
app.add_url_rule(rule=routes.UPDATE_GROUP_PROMPT_PERMISSION, methods=["PATCH"], view_func=views.update_group_prompt_permission)

# regex permission management
app.add_url_rule(
    rule=routes.CREATE_EXPERIMENT_REGEX_PERMISSION, methods=["POST"], view_func=views.create_experiment_regex_permission
)
app.add_url_rule(rule=routes.GET_EXPERIMENT_REGEX_PERMISSION, methods=["GET"], view_func=views.get_experiment_regex_permission)
app.add_url_rule(
    rule=routes.UPDATE_EXPERIMENT_REGEX_PERMISSION, methods=["PATCH"], view_func=views.update_experiment_regex_permission
)
app.add_url_rule(
    rule=routes.DELETE_EXPERIMENT_REGEX_PERMISSION, methods=["DELETE"], view_func=views.delete_experiment_regex_permission
)
app.add_url_rule(
    rule=routes.CREATE_REGISTERED_MODEL_REGEX_PERMISSION,
    methods=["POST"],
    view_func=views.create_registered_model_regex_permission,
)
app.add_url_rule(
    rule=routes.GET_REGISTERED_MODEL_REGEX_PERMISSION, methods=["GET"], view_func=views.get_registered_model_regex_permission
)
app.add_url_rule(
    rule=routes.UPDATE_REGISTERED_MODEL_REGEX_PERMISSION,
    methods=["PATCH"],
    view_func=views.update_registered_model_regex_permission,
)
app.add_url_rule(
    rule=routes.DELETE_REGISTERED_MODEL_REGEX_PERMISSION,
    methods=["DELETE"],
    view_func=views.delete_registered_model_regex_permission,
)

app.add_url_rule(rule=routes.CREATE_PROMPT_REGEX_PERMISSION, methods=["POST"], view_func=views.create_prompt_regex_permission)
app.add_url_rule(rule=routes.GET_PROMPT_REGEX_PERMISSION, methods=["GET"], view_func=views.get_prompt_regex_permission)
app.add_url_rule(rule=routes.UPDATE_PROMPT_REGEX_PERMISSION, methods=["PATCH"], view_func=views.update_prompt_regex_permission)
app.add_url_rule(rule=routes.DELETE_PROMPT_REGEX_PERMISSION, methods=["DELETE"], view_func=views.delete_prompt_regex_permission)

app.add_url_rule(
    rule=routes.CREATE_USER_EXPERIMENT_REGEX_PERMISSION,
    methods=["POST"],
    view_func=views.create_user_experiment_regex_permission,
)

app.add_url_rule(
    rule=routes.GET_USER_EXPERIMENT_REGEX_PERMISSION,
    methods=["GET"],
    view_func=views.get_user_experiment_regex_permission,
)

app.add_url_rule(
    rule=routes.UPDATE_USER_EXPERIMENT_REGEX_PERMISSION,
    methods=["PATCH"],
    view_func=views.update_user_experiment_regex_permission,
)

app.add_url_rule(
    rule=routes.DELETE_USER_EXPERIMENT_REGEX_PERMISSION,
    methods=["DELETE"],
    view_func=views.delete_user_experiment_regex_permission,
)

app.add_url_rule(
    rule=routes.CREATE_USER_REGISTERED_MODEL_REGEX_PERMISSION,
    methods=["POST"],
    view_func=views.create_user_registered_model_regex_permission,
)

app.add_url_rule(
    rule=routes.GET_USER_REGISTERED_MODEL_REGEX_PERMISSION,
    methods=["GET"],
    view_func=views.get_user_registered_model_regex_permission,
)

app.add_url_rule(
    rule=routes.UPDATE_USER_REGISTERED_MODEL_REGEX_PERMISSION,
    methods=["PATCH"],
    view_func=views.update_user_registered_model_regex_permission,
)

app.add_url_rule(
    rule=routes.DELETE_USER_REGISTERED_MODEL_REGEX_PERMISSION,
    methods=["DELETE"],
    view_func=views.delete_user_registered_model_regex_permission,
)

app.add_url_rule(
    rule=routes.CREATE_USER_PROMPT_REGEX_PERMISSION,
    methods=["POST"],
    view_func=views.create_user_prompt_regex_permission,
)

app.add_url_rule(
    rule=routes.GET_USER_PROMPT_REGEX_PERMISSION,
    methods=["GET"],
    view_func=views.get_user_prompt_regex_permission,
)

app.add_url_rule(
    rule=routes.UPDATE_USER_PROMPT_REGEX_PERMISSION,
    methods=["PATCH"],
    view_func=views.update_user_prompt_regex_permission,
)

app.add_url_rule(
    rule=routes.DELETE_USER_PROMPT_REGEX_PERMISSION,
    methods=["DELETE"],
    view_func=views.delete_user_prompt_regex_permission,
)

app.add_url_rule(
    rule=routes.CREATE_GROUP_EXPERIMENT_REGEX_PERMISSION,
    methods=["POST"],
    view_func=views.create_group_experiment_regex_permission,
)
app.add_url_rule(
    rule=routes.GET_GROUP_EXPERIMENT_REGEX_PERMISSION,
    methods=["GET"],
    view_func=views.get_group_experiment_regex_permission,
)
app.add_url_rule(
    rule=routes.UPDATE_GROUP_EXPERIMENT_REGEX_PERMISSION,
    methods=["PATCH"],
    view_func=views.update_group_experiment_regex_permission,
)
app.add_url_rule(
    rule=routes.DELETE_GROUP_EXPERIMENT_REGEX_PERMISSION,
    methods=["DELETE"],
    view_func=views.delete_group_experiment_regex_permission,
)
app.add_url_rule(
    rule=routes.CREATE_GROUP_REGISTERED_MODEL_REGEX_PERMISSION,
    methods=["POST"],
    view_func=views.create_group_registered_model_regex_permission,
)
app.add_url_rule(
    rule=routes.GET_GROUP_REGISTERED_MODEL_REGEX_PERMISSION,
    methods=["GET"],
    view_func=views.get_group_registered_model_regex_permission,
)
app.add_url_rule(
    rule=routes.UPDATE_GROUP_REGISTERED_MODEL_REGEX_PERMISSION,
    methods=["PATCH"],
    view_func=views.update_group_registered_model_regex_permission,
)
app.add_url_rule(
    rule=routes.DELETE_GROUP_REGISTERED_MODEL_REGEX_PERMISSION,
    methods=["DELETE"],
    view_func=views.delete_group_registered_model_regex_permission,
)

app.add_url_rule(
    rule=routes.CREATE_GROUP_PROMPT_REGEX_PERMISSION,
    methods=["POST"],
    view_func=views.create_group_prompt_regex_permission,
)
app.add_url_rule(
    rule=routes.GET_GROUP_PROMPT_REGEX_PERMISSION,
    methods=["GET"],
    view_func=views.get_group_prompt_regex_permission,
)
app.add_url_rule(
    rule=routes.UPDATE_GROUP_PROMPT_REGEX_PERMISSION,
    methods=["PATCH"],
    view_func=views.update_group_prompt_regex_permission,
)
app.add_url_rule(
    rule=routes.DELETE_GROUP_PROMPT_REGEX_PERMISSION,
    methods=["DELETE"],
    view_func=views.delete_group_prompt_regex_permission,
)

# Add new hooks
app.before_request(before_request_hook)
app.after_request(after_request_hook)

# Set up session
Session(app)
cache = Cache(app)



from typing import List, Callable

from mlflow.exceptions import MlflowException
from mlflow.protos.databricks_pb2 import RESOURCE_ALREADY_EXISTS
from sqlalchemy.exc import IntegrityError

from mlflow_oidc_auth.db.models import SqlRegisteredModelRegexPermission, SqlUser
from mlflow_oidc_auth.entities import RegisteredModelRegexPermission
from mlflow_oidc_auth.permissions import _validate_permission
from mlflow_oidc_auth.repository.utils import get_one_or_raise, get_user, validate_regex, get_all
from sqlalchemy.orm import Session


class RegisteredModelPermissionRegexRepository:
    def __init__(self, session_maker):
        self._Session: Callable[[], Session] = session_maker

    def grant(
        self,
        regex: str,
        priority: int,
        permission: str,
        username: str,
        prompt: bool = False,
    ) -> RegisteredModelRegexPermission:
        validate_regex(regex)
        _validate_permission(permission)
        with self._Session() as session:
            try:
                user = get_user(session, username)
                perm = SqlRegisteredModelRegexPermission(
                    regex=regex,
                    priority=priority,
                    user_id=user.id,
                    permission=permission,
                    prompt=prompt,
                )
                session.add(perm)
                session.flush()
                return perm.to_mlflow_entity()
            except IntegrityError as e:
                raise MlflowException(
                    f"Registered model perm exists ({regex},{username}): {e}",
                    RESOURCE_ALREADY_EXISTS,
                )

    def get(self, regex: str, username: str, prompt: bool = False) -> RegisteredModelRegexPermission:
        with self._Session() as session:
            user = get_user(session, username)
            perm: SqlRegisteredModelRegexPermission = get_one_or_raise(
                session,
                SqlRegisteredModelRegexPermission,
                SqlRegisteredModelRegexPermission.regex == regex,
                SqlRegisteredModelRegexPermission.user_id == user.id,
                SqlRegisteredModelRegexPermission.prompt == prompt,
                not_found_msg=f"No registered model perm for regex={regex}, user={username}",
                multiple_msg=f"Multiple registered model perms for regex={regex}, user={username}",
            )
            return perm.to_mlflow_entity()

    def list_regex_for_user(self, username: str, prompt: bool = False) -> List[RegisteredModelRegexPermission]:
        with self._Session() as session:
            user = get_user(session, username)
            perms = get_all(
                session,
                SqlRegisteredModelRegexPermission,
                SqlRegisteredModelRegexPermission.user_id == user.id,
                SqlRegisteredModelRegexPermission.prompt == prompt,
                order_by=SqlRegisteredModelRegexPermission.priority,
            )
            return [p.to_mlflow_entity() for p in perms]

    def update(
        self, regex: str, priority: int, permission: str, username: str, prompt: bool = False
    ) -> RegisteredModelRegexPermission:
        validate_regex(regex)
        _validate_permission(permission)
        with self._Session() as session:
            user = get_user(session, username)
            perm: SqlRegisteredModelRegexPermission = get_one_or_raise(
                session,
                SqlRegisteredModelRegexPermission,
                SqlRegisteredModelRegexPermission.regex == regex,
                SqlRegisteredModelRegexPermission.user_id == user.id,
                SqlRegisteredModelRegexPermission.prompt == prompt,
                not_found_msg=f"No registered model perm to update for regex={regex}, user={username}",
                multiple_msg=f"Multiple registered model perms for regex={regex}, user={username}",
            )
            perm.priority = priority
            perm.permission = permission
            session.commit()
            return perm.to_mlflow_entity()

    def revoke(self, regex: str, username: str, prompt: bool = False) -> None:
        with self._Session() as session:
            user = get_user(session, username)
            perm = get_one_or_raise(
                session,
                SqlRegisteredModelRegexPermission,
                SqlRegisteredModelRegexPermission.regex == regex,
                SqlRegisteredModelRegexPermission.user_id == user.id,
                SqlRegisteredModelRegexPermission.prompt == prompt,
                not_found_msg=f"No registered model perm to delete for regex={regex}, user={username}",
                multiple_msg=f"Multiple registered model perms for regex={regex}, user={username}",
            )
            session.delete(perm)
            session.commit()
            return None



from mlflow_oidc_auth.repository.experiment_permission import ExperimentPermissionRepository
from mlflow_oidc_auth.repository.experiment_permission_group import ExperimentPermissionGroupRepository
from mlflow_oidc_auth.repository.group import GroupRepository
from mlflow_oidc_auth.repository.prompt_permission_group import PromptPermissionGroupRepository
from mlflow_oidc_auth.repository.registered_model_permission import RegisteredModelPermissionRepository
from mlflow_oidc_auth.repository.registered_model_permission_group import RegisteredModelPermissionGroupRepository
from mlflow_oidc_auth.repository.user import UserRepository
from mlflow_oidc_auth.repository.experiment_permission_regex import ExperimentPermissionRegexRepository
from mlflow_oidc_auth.repository.experiment_permission_regex_group import ExperimentPermissionGroupRegexRepository
from mlflow_oidc_auth.repository.registered_model_permission_regex import RegisteredModelPermissionRegexRepository
from mlflow_oidc_auth.repository.registered_model_permission_regex_group import RegisteredModelGroupRegexPermissionRepository



from typing import List, Callable
from sqlalchemy.orm import Session

from mlflow.exceptions import MlflowException
from mlflow.protos.databricks_pb2 import RESOURCE_ALREADY_EXISTS
from sqlalchemy.exc import IntegrityError

from mlflow_oidc_auth.db.models import SqlRegisteredModelPermission, SqlUser
from mlflow_oidc_auth.entities import RegisteredModelPermission
from mlflow_oidc_auth.permissions import _validate_permission
from mlflow_oidc_auth.repository.utils import get_one_or_raise, get_user


class RegisteredModelPermissionRepository:
    def __init__(self, session_maker):
        self._Session: Callable[[], Session] = session_maker

    def create(self, name: str, username: str, permission: str) -> RegisteredModelPermission:
        _validate_permission(permission)
        with self._Session() as session:
            try:
                user = get_user(session, username)
                perm = SqlRegisteredModelPermission(name=name, user_id=user.id, permission=permission)
                session.add(perm)
                session.flush()
                return perm.to_mlflow_entity()
            except IntegrityError as e:
                raise MlflowException(
                    f"Registered‑model perm exists ({name},{username}): {e}",
                    RESOURCE_ALREADY_EXISTS,
                )

    def get(self, name: str, username: str) -> RegisteredModelPermission:
        with self._Session() as session:
            row = get_one_or_raise(
                session,
                SqlRegisteredModelPermission,
                SqlRegisteredModelPermission.name == name,
                SqlRegisteredModelPermission.user_id
                == session.query(SqlUser.id).filter(SqlUser.username == username).scalar_subquery(),
                not_found_msg=f"No model perm for name={name}, user={username}",
                multiple_msg=f"Multiple model perms for name={name}, user={username}",
            )
            return row.to_mlflow_entity()

    def list_for_user(self, username: str) -> List[RegisteredModelPermission]:
        with self._Session() as session:
            user = get_user(session, username)
            rows = session.query(SqlRegisteredModelPermission).filter(SqlRegisteredModelPermission.user_id == user.id).all()
            return [r.to_mlflow_entity() for r in rows]

    def update(self, name: str, username: str, permission: str) -> RegisteredModelPermission:
        _validate_permission(permission)
        with self._Session() as session:
            perm = get_one_or_raise(
                session,
                SqlRegisteredModelPermission,
                SqlRegisteredModelPermission.name == name,
                SqlRegisteredModelPermission.user_id
                == session.query(SqlUser.id).filter(SqlUser.username == username).scalar_subquery(),
                not_found_msg=f"No perm to update for name={name}, user={username}",
                multiple_msg=f"Multiple perms for name={name}, user={username}",
            )
            perm.permission = permission
            session.flush()
            return perm.to_mlflow_entity()

    def delete(self, name: str, username: str) -> None:
        with self._Session() as session:
            perm = get_one_or_raise(
                session,
                SqlRegisteredModelPermission,
                SqlRegisteredModelPermission.name == name,
                SqlRegisteredModelPermission.user_id
                == session.query(SqlUser.id).filter(SqlUser.username == username).scalar_subquery(),
                not_found_msg=f"No perm to delete for name={name}, user={username}",
                multiple_msg=f"Multiple perms for name={name}, user={username}",
            )
            session.delete(perm)
            session.flush()

    def wipe(self, name: str):
        with self._Session() as session:
            perms = session.query(SqlRegisteredModelPermission).filter(SqlRegisteredModelPermission.name == name).all()
            for p in perms:
                session.delete(p)
            session.flush()



from unittest import mock
from mlflow_oidc_auth import routes

"""
`routes` contains mutiple routes definitions.
This test is to ensure that all expected routes are present.
"""


class TestRoutes:
    def test_routes_presented(self):
        assert all(
            route is not None
            for route in [
                routes.HOME,
                routes.LOGIN,
                routes.LOGOUT,
                routes.CALLBACK,
                routes.STATIC,
                routes.UI,
                routes.UI_ROOT,
                routes.CREATE_ACCESS_TOKEN,
                routes.GET_CURRENT_USER,
                routes.GET_EXPERIMENTS,
                routes.GET_MODELS,
                routes.GET_USERS,
                routes.GET_USER_EXPERIMENTS,
                routes.GET_USER_MODELS,
                routes.GET_EXPERIMENT_USERS,
                routes.GET_MODEL_USERS,
                routes.CREATE_USER,
                routes.GET_USER,
                routes.UPDATE_USER_PASSWORD,
                routes.UPDATE_USER_ADMIN,
                routes.DELETE_USER,
                routes.CREATE_EXPERIMENT_PERMISSION,
                routes.GET_EXPERIMENT_PERMISSION,
                routes.UPDATE_EXPERIMENT_PERMISSION,
                routes.DELETE_EXPERIMENT_PERMISSION,
                routes.CREATE_REGISTERED_MODEL_PERMISSION,
                routes.GET_REGISTERED_MODEL_PERMISSION,
                routes.UPDATE_REGISTERED_MODEL_PERMISSION,
                routes.DELETE_REGISTERED_MODEL_PERMISSION,
                routes.GET_GROUPS,
                routes.GET_GROUP_USERS,
                routes.GET_GROUP_EXPERIMENTS_PERMISSION,
                routes.CREATE_GROUP_EXPERIMENT_PERMISSION,
                routes.DELETE_GROUP_EXPERIMENT_PERMISSION,
                routes.UPDATE_GROUP_EXPERIMENT_PERMISSION,
                routes.GET_GROUP_MODELS_PERMISSION,
                routes.CREATE_GROUP_MODEL_PERMISSION,
                routes.DELETE_GROUP_MODEL_PERMISSION,
                routes.UPDATE_GROUP_MODEL_PERMISSION,
            ]
        )



import pytest
from unittest.mock import MagicMock, patch
from mlflow_oidc_auth.repository.experiment_permission_regex import ExperimentPermissionRegexRepository
from mlflow_oidc_auth.entities import ExperimentRegexPermission
from mlflow.exceptions import MlflowException
from mlflow.protos.databricks_pb2 import RESOURCE_ALREADY_EXISTS


@pytest.fixture
def session():
    s = MagicMock()
    s.__enter__.return_value = s
    s.__exit__.return_value = None
    return s


@pytest.fixture
def session_maker(session):
    return MagicMock(return_value=session)


@pytest.fixture
def repo(session_maker):
    return ExperimentPermissionRegexRepository(session_maker)


def test_grant_integrity_error(repo, session):
    user = MagicMock(id=2)
    session.add = MagicMock()
    session.flush = MagicMock(side_effect=Exception("IntegrityError"))
    with patch("mlflow_oidc_auth.repository.experiment_permission_regex.get_user", return_value=user), patch(
        "mlflow_oidc_auth.db.models.SqlExperimentRegexPermission", return_value=MagicMock()
    ), patch("mlflow_oidc_auth.repository.experiment_permission_regex.IntegrityError", Exception):
        with pytest.raises(MlflowException):
            repo.grant("r", 1, "READ", "user")


def test_get(repo, session):
    row = MagicMock()
    row.to_mlflow_entity.return_value = "entity"
    with patch("mlflow_oidc_auth.repository.experiment_permission_regex.get_one_or_raise", return_value=row):
        session.query().filter().scalar.return_value = 1
        assert repo.get("r", "user") == "entity"


def test_list(repo, session):
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    session.query().all.return_value = [perm]
    assert repo.list() == ["entity"]


def test_list_regex_for_user(repo, session):
    user = MagicMock(id=3)
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    session.query().filter().all.return_value = [perm]
    with patch("mlflow_oidc_auth.repository.experiment_permission_regex.get_user", return_value=user):
        assert repo.list_regex_for_user("user") == ["entity"]


def test_update(repo, session):
    perm = MagicMock()
    perm.to_mlflow_entity.return_value = "entity"
    with patch("mlflow_oidc_auth.repository.experiment_permission_regex.get_one_or_raise", return_value=perm):
        session.flush = MagicMock()
        result = repo.update("r", 2, "EDIT", "user")
        assert result == "entity"
        assert perm.priority == 2
        assert perm.permission == "EDIT"
        session.flush.assert_called_once()


def test_revoke(repo, session):
    perm = MagicMock()
    with patch("mlflow_oidc_auth.repository.experiment_permission_regex.get_one_or_raise", return_value=perm):
        session.delete = MagicMock()
        session.commit = MagicMock()
        assert repo.revoke("r", "user") is None
        session.delete.assert_called_once_with(perm)
        session.commit.assert_called_once()



import pytest
from unittest.mock import patch
from mlflow.exceptions import RestException
from mlflow_oidc_auth.client import AuthServiceClient
from mlflow_oidc_auth.routes import (
    CREATE_EXPERIMENT_PERMISSION,
    CREATE_REGISTERED_MODEL_PERMISSION,
    CREATE_USER,
    DELETE_EXPERIMENT_PERMISSION,
    DELETE_REGISTERED_MODEL_PERMISSION,
    DELETE_USER,
    GET_EXPERIMENT_PERMISSION,
    GET_REGISTERED_MODEL_PERMISSION,
    GET_USER,
    UPDATE_EXPERIMENT_PERMISSION,
    UPDATE_REGISTERED_MODEL_PERMISSION,
    UPDATE_USER_ADMIN,
    UPDATE_USER_PASSWORD,
)


@pytest.fixture
def client():
    return AuthServiceClient(tracking_uri="https://test")


class TestClientExceptions:
    def test_create_user_no_password(self, client):
        with pytest.raises(ValueError):
            client.create_user("test_user", None)

    def test_create_user_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.create_user(None, "password")

    def test_get_user_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.get_user(None)

    def test_update_user_password_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.update_user_password(None, "password")

    def test_update_user_password_with_no_password(self, client):
        with pytest.raises(ValueError):
            client.update_user_password("test_user", None)

    def test_update_user_admin_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.update_user_admin(None, True)

    def test_update_user_admin_with_no_boolean(self, client):
        # TODO: probably, use mypy to check typing
        with pytest.raises(ValueError):
            client.update_user_admin("test_user", "this is not boolean value")

    def test_delete_user_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.delete_user(None)

    def test_create_experiment_permission_with_no_experiment_id(self, client):
        with pytest.raises(ValueError):
            client.create_experiment_permission(None, "test_user", "READ")

    def test_create_experiment_permission_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.create_experiment_permission("1", None, "READ")

    def test_create_experiment_permission_with_invalid_permission(self, client):
        with pytest.raises(ValueError):
            client.create_experiment_permission("1", "test_user", "THIS_PERMISSION_DOES_NOT_EXIST")

    def test_get_experiment_permission_with_no_experiment_id(self, client):
        with pytest.raises(ValueError):
            client.get_experiment_permission(None, "test_user")

    def test_get_experiment_permission_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.get_experiment_permission("1", None)

    def test_update_experiment_permission_with_no_experiment_id(self, client):
        with pytest.raises(ValueError):
            client.update_experiment_permission(None, "test_user", "READ")

    def test_update_experiment_permission_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.update_experiment_permission("1", None, "READ")

    def test_update_experiment_permission_with_invalid_permission(self, client):
        with pytest.raises(ValueError):
            client.update_experiment_permission("1", "test_user", "THIS_PERMISSION_DOES_NOT_EXIST")

    def test_delete_experiment_permission_with_no_experiment_id(self, client):
        with pytest.raises(ValueError):
            client.delete_experiment_permission(None, "test_user")

    def test_delete_experiment_permission_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.delete_experiment_permission("1", None)

    def test_create_registered_model_permission_with_no_name(self, client):
        with pytest.raises(ValueError):
            client.create_registered_model_permission(None, "test_user", "READ")

    def test_create_registered_model_permission_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.create_registered_model_permission("model", None, "READ")

    def test_create_registered_model_permission_with_invalid_permission(self, client):
        with pytest.raises(ValueError):
            client.create_registered_model_permission("model", "test_user", "THIS_PERMISSION_DOES_NOT_EXIST")

    def test_get_registered_model_permission_with_no_name(self, client):
        with pytest.raises(ValueError):
            client.get_registered_model_permission(None, "test_user")

    def test_get_registered_model_permission_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.get_registered_model_permission("model", None)

    def test_update_registered_model_permission_with_no_name(self, client):
        with pytest.raises(ValueError):
            client.update_registered_model_permission(None, "test_user", "READ")

    def test_update_registered_model_permission_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.update_registered_model_permission("model", None, "READ")

    def test_update_registered_model_permission_with_invalid_permission(self, client):
        with pytest.raises(ValueError):
            client.update_registered_model_permission("model", "test_user", "THIS_PERMISSION_DOES_NOT_EXIST")

    def test_delete_registered_model_permission_with_no_name(self, client):
        with pytest.raises(ValueError):
            client.delete_registered_model_permission(None, "test_user")

    def test_delete_registered_model_permission_with_no_username(self, client):
        with pytest.raises(ValueError):
            client.delete_registered_model_permission("model", None)


@pytest.fixture(
    scope="function",
    autouse=True,
)
def mock_request():
    with patch("mlflow_oidc_auth.client.AuthServiceClient._request") as mock:
        yield mock


class TestClient:
    def test_create_user(self, mock_request, client):
        mock_request.return_value = {
            "user": {
                "id": 123,
                "username": "test_user",
                "display_name": "Test User",
                "is_admin": False,
                "experiment_permissions": [],
                "registered_model_permissions": [],
                "groups": [],
            }
        }
        user = client.create_user("test_user", "password")
        assert user.username == "test_user"
        mock_request.assert_called_once_with(CREATE_USER, "POST", json={"username": "test_user", "password": "password"})

    def test_get_user(self, mock_request, client):
        mock_request.return_value = {
            "user": {
                "id": 123,
                "username": "test_user",
                "display_name": "Test User",
                "is_admin": False,
                "experiment_permissions": [],
                "registered_model_permissions": [],
                "groups": [],
            }
        }
        user = client.get_user("test_user")
        assert user.username == "test_user"
        mock_request.assert_called_once_with(GET_USER, "GET", params={"username": "test_user"})

    def test_update_user_password(self, mock_request, client):
        client.update_user_password("test_user", "new_password")
        mock_request.assert_called_once_with(
            UPDATE_USER_PASSWORD, "PATCH", json={"username": "test_user", "password": "new_password"}
        )

    def test_update_user_admin(self, mock_request, client):
        mock_request.return_value = {
            "user": {
                "id": 123,
                "username": "test_user",
                "display_name": "Test User",
                "is_admin": True,
                "experiment_permissions": [],
                "registered_model_permissions": [],
                "groups": [],
            }
        }
        client.update_user_admin("test_user", True)
        mock_request.assert_called_once_with(UPDATE_USER_ADMIN, "PATCH", json={"username": "test_user", "is_admin": True})

    def test_delete_user(self, mock_request, client):
        client.delete_user("test_user")
        mock_request.assert_called_once_with(DELETE_USER, "DELETE", json={"username": "test_user"})

    def test_create_experiment_permission(self, mock_request, client):
        mock_request.return_value = {"experiment_permission": {"experiment_id": 1, "permission": "READ", "user_id": 123}}
        permission = client.create_experiment_permission("1", "test_user", "READ")
        assert permission.permission == "READ"
        assert permission.experiment_id == 1
        assert permission.user_id == 123
        mock_request.assert_called_once_with(
            CREATE_EXPERIMENT_PERMISSION, "POST", json={"experiment_id": "1", "username": "test_user", "permission": "READ"}
        )

    def test_get_experiment_permission(self, mock_request, client):
        mock_request.return_value = {"experiment_permission": {"experiment_id": 1, "permission": "READ", "user_id": 123}}
        permission = client.get_experiment_permission("1", "test_user")
        assert permission.permission == "READ"
        assert permission.experiment_id == 1
        assert permission.user_id == 123
        mock_request.assert_called_once_with(
            GET_EXPERIMENT_PERMISSION, "GET", params={"experiment_id": "1", "username": "test_user"}
        )

    def test_update_experiment_permission(self, mock_request, client):
        mock_request.return_value = {"experiment_permission": {"experiment_id": 1, "permission": "READ", "user_id": 123}}
        client.update_experiment_permission("1", "test_user", "READ")
        mock_request.assert_called_once_with(
            UPDATE_EXPERIMENT_PERMISSION, "PATCH", json={"experiment_id": "1", "username": "test_user", "permission": "READ"}
        )

    def test_delete_experiment_permission(self, mock_request, client):
        client.delete_experiment_permission("1", "test_user")
        mock_request.assert_called_once_with(
            DELETE_EXPERIMENT_PERMISSION, "DELETE", json={"experiment_id": "1", "username": "test_user"}
        )

    def test_create_registered_model_permission(self, mock_request, client):
        mock_request.return_value = {"registered_model_permission": {"name": "model", "permission": "READ", "user_id": 123}}
        permission = client.create_registered_model_permission("model", "test_user", "READ")
        assert permission.permission == "READ"
        assert permission.name == "model"
        assert permission.user_id == 123
        mock_request.assert_called_once_with(
            CREATE_REGISTERED_MODEL_PERMISSION, "POST", json={"name": "model", "username": "test_user", "permission": "READ"}
        )

    def test_get_registered_model_permission(self, mock_request, client):
        mock_request.return_value = {"registered_model_permission": {"name": "model", "permission": "READ", "user_id": 123}}
        permission = client.get_registered_model_permission("model", "test_user")
        assert permission.permission == "READ"
        assert permission.name == "model"
        assert permission.user_id == 123
        mock_request.assert_called_once_with(
            GET_REGISTERED_MODEL_PERMISSION, "GET", params={"name": "model", "username": "test_user"}
        )

    def test_update_registered_model_permission(self, mock_request, client):
        mock_request.return_value = {"registered_model_permission": {"name": "model", "permission": "READ", "user_id": 123}}
        client.update_registered_model_permission("model", "test_user", "READ")
        mock_request.assert_called_once_with(
            UPDATE_REGISTERED_MODEL_PERMISSION, "PATCH", json={"name": "model", "username": "test_user", "permission": "READ"}
        )

    def test_delete_registered_model_permission(self, mock_request, client):
        client.delete_registered_model_permission("model", "test_user")
        mock_request.assert_called_once_with(
            DELETE_REGISTERED_MODEL_PERMISSION, "DELETE", json={"name": "model", "username": "test_user"}
        )

    def test_request(self, mock_request, client):
        mock_request.return_value = {"key": "value"}
        response = client._request("endpoint", "GET")
        assert response == {"key": "value"}
        mock_request.assert_called_once()

    def test_request_with_kwargs(self, mock_request, client):
        mock_request.return_value = {"key": "value"}
        response = client._request("endpoint", "POST", json={"data": "value"})
        assert response == {"key": "value"}
        mock_request.assert_called_once_with("endpoint", "POST", json={"data": "value"})

    def test_create_user_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.create_user("test_user", "password")

    def test_get_user_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.get_user("test_user")

    def test_update_user_password_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.update_user_password("test_user", "new_password")

    def test_update_user_admin_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.update_user_admin("test_user", True)

    def test_delete_user_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.delete_user("test_user")

    def test_create_experiment_permission_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.create_experiment_permission("1", "test_user", "READ")

    def test_get_experiment_permission_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.get_experiment_permission("1", "test_user")

    def test_update_experiment_permission_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.update_experiment_permission("1", "test_user", "READ")

    def test_delete_experiment_permission_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.delete_experiment_permission("1", "test_user")

    def test_create_registered_model_permission_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.create_registered_model_permission("model", "test_user", "READ")

    def test_get_registered_model_permission_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.get_registered_model_permission("model", "test_user")

    def test_update_registered_model_permission_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.update_registered_model_permission("model", "test_user", "READ")

    def test_delete_registered_model_permission_rest_exception(self, mock_request, client):
        mock_request.side_effect = RestException({"error_code": "INTERNAL_ERROR", "message": "Error"})
        with pytest.raises(RestException):
            client.delete_registered_model_permission("model", "test_user")



import importlib
from unittest.mock import MagicMock, patch

import pytest

from mlflow_oidc_auth.auth import (
    _get_oidc_jwks,
    authenticate_request_basic_auth,
    authenticate_request_bearer_token,
    get_oauth_instance,
    validate_token,
)


class TestAuth:
    @patch("mlflow_oidc_auth.auth.OAuth")
    @patch("mlflow_oidc_auth.auth.config")
    def test_get_oauth_instance(self, mock_config, mock_oauth):
        mock_app = MagicMock()
        mock_oauth_instance = MagicMock()
        mock_oauth.return_value = mock_oauth_instance

        mock_config.OIDC_CLIENT_ID = "mock_client_id"
        mock_config.OIDC_CLIENT_SECRET = "mock_client_secret"
        mock_config.OIDC_DISCOVERY_URL = "mock_discovery_url"
        mock_config.OIDC_SCOPE = "mock_scope"

        result = get_oauth_instance(mock_app)

        mock_oauth.assert_called_once_with(mock_app)
        mock_oauth_instance.register.assert_called_once_with(
            name="oidc",
            client_id="mock_client_id",
            client_secret="mock_client_secret",
            server_metadata_url="mock_discovery_url",
            client_kwargs={"scope": "mock_scope"},
        )
        assert result == mock_oauth_instance

    @patch("mlflow_oidc_auth.auth.requests")
    @patch("mlflow_oidc_auth.auth.config")
    def test__get_oidc_jwks(self, mock_config, mock_requests):
        mock_cache = MagicMock()
        mock_app = MagicMock()
        mock_app.logger.debug = MagicMock()
        mock_requests.get.return_value.json.return_value = {"jwks_uri": "mock_jwks_uri"}
        mock_cache.get.return_value = None
        mock_config.OIDC_DISCOVERY_URL = "mock_discovery_url"

        # cache and app are imported within the _get_oidc_jwks function
        mlflow_oidc_app = importlib.import_module("mlflow_oidc_auth.app")
        with patch.object(mlflow_oidc_app, "cache", mock_cache):
            with patch.object(mlflow_oidc_app, "app", mock_app):
                result = _get_oidc_jwks()

                assert len(mock_requests.get.call_args) == 2

                assert mock_requests.get.call_args[0][0] == "mock_jwks_uri"
                assert mock_requests.get.call_args[1] == {}  # TODO: proper patch for first .get() return_value

                mock_cache.set.assert_called_once_with("jwks", mock_requests.get.return_value.json.return_value, timeout=3600)
                assert result == mock_requests.get.return_value.json.return_value

    @patch("mlflow_oidc_auth.auth.app")
    def test__get_oidc_jwks_cache_hit(self, mock_app):
        mock_cache = MagicMock()
        mock_cache.get.return_value = {"keys": "mock_keys"}
        mock_app.logger.debug = MagicMock()
        mlflow_oidc_app = importlib.import_module("mlflow_oidc_auth.app")
        with patch.object(mlflow_oidc_app, "cache", mock_cache):
            result = _get_oidc_jwks()
            mock_app.logger.debug.assert_called_once_with("JWKS cache hit")
            assert result == {"keys": "mock_keys"}

    @patch("mlflow_oidc_auth.auth.config")
    def test__get_oidc_jwks_no_discovery_url(self, mock_config):
        mock_config.OIDC_DISCOVERY_URL = None
        with pytest.raises(ValueError, match="OIDC_DISCOVERY_URL is not set in the configuration"):
            _get_oidc_jwks()

    @patch("mlflow_oidc_auth.auth._get_oidc_jwks")
    @patch("mlflow_oidc_auth.auth.jwt.decode")
    def test_validate_token(self, mock_jwt_decode, mock_get_oidc_jwks):
        mock_jwks = {"keys": "mock_keys"}
        mock_get_oidc_jwks.return_value = mock_jwks
        mock_payload = MagicMock()
        mock_jwt_decode.return_value = mock_payload

        token = "mock_token"
        result = validate_token(token)

        mock_get_oidc_jwks.assert_called_once()
        mock_jwt_decode.assert_called_once_with(token, mock_jwks)
        mock_payload.validate.assert_called_once()
        assert result == mock_payload

    @patch("mlflow_oidc_auth.auth.store")
    def test_authenticate_request_basic_auth_uses_authenticate_user(self, mock_store):
        mock_request = MagicMock()
        mock_request.authorization.username = "mock_username"
        mock_request.authorization.password = "mock_password"
        mock_store.authenticate_user.return_value = True

        with patch("mlflow_oidc_auth.auth.request", mock_request):
            # for some reason decorator doesn't mock flask
            result = authenticate_request_basic_auth()

            mock_store.authenticate_user.assert_called_once_with("mock_username", "mock_password")
            assert result == True

    def test_authenticate_request_basic_auth_no_authorization(self):
        mock_request = MagicMock()
        mock_request.authorization = None
        with patch("mlflow_oidc_auth.auth.request", mock_request):
            result = authenticate_request_basic_auth()
            assert result == False

    @patch("mlflow_oidc_auth.auth.store")
    @patch("mlflow_oidc_auth.auth.app")
    def test_authenticate_request_basic_auth_user_not_authenticated(self, mock_app, mock_store):
        mock_request = MagicMock()
        mock_request.authorization.username = "mock_username"
        mock_request.authorization.password = "mock_password"
        mock_store.authenticate_user.return_value = False
        mock_app.logger.debug = MagicMock()

        with patch("mlflow_oidc_auth.auth.request", mock_request):
            result = authenticate_request_basic_auth()
            mock_app.logger.debug.assert_called_with("User %s not authenticated", "mock_username")
            assert result == False

    @patch("mlflow_oidc_auth.auth.validate_token")
    def test_authenticate_request_bearer_token_uses_validate_token(self, mock_validate_token):
        mock_request = MagicMock()
        mock_request.authorization.token = "mock_token"
        mock_validate_token.return_value = MagicMock()
        with patch("mlflow_oidc_auth.auth.request", mock_request):
            # for some reason decorator doesn't mock flask
            result = authenticate_request_bearer_token()

            mock_validate_token.assert_called_once_with("mock_token")
            assert result == True

    @patch("mlflow_oidc_auth.auth.validate_token")
    def test_authenticate_request_bearer_token_exception_returns_false(self, mock_validate_token):
        mock_request = MagicMock()
        mock_request.authorization.token = "mock_token"
        mock_validate_token.side_effect = Exception()
        with patch("mlflow_oidc_auth.auth.request", mock_request):
            # for some reason decorator doesn't mock flask
            result = authenticate_request_bearer_token()

            mock_validate_token.assert_called_once_with("mock_token")
            assert result == False

    def test_authenticate_request_bearer_token_no_authorization(self):
        mock_request = MagicMock()
        mock_request.authorization = None
        with patch("mlflow_oidc_auth.auth.request", mock_request):
            result = authenticate_request_bearer_token()
            assert result == False

    def test_authenticate_request_bearer_token_no_token(self):
        mock_request = MagicMock()
        mock_request.authorization.token = None
        with patch("mlflow_oidc_auth.auth.request", mock_request):
            result = authenticate_request_bearer_token()
            assert result == False



from mlflow.server.handlers import _get_tracking_store

from mlflow_oidc_auth.permissions import Permission
from mlflow_oidc_auth.utils import effective_experiment_permission, get_request_param, get_username


def _get_permission_from_run_id() -> Permission:
    # run permissions inherit from parent resource (experiment)
    # so we just get the experiment permission
    run_id = get_request_param("run_id")
    run = _get_tracking_store().get_run(run_id)
    experiment_id = run.info.experiment_id
    username = get_username()
    return effective_experiment_permission(experiment_id, username).permission


def validate_can_read_run():
    return _get_permission_from_run_id().can_read


def validate_can_update_run():
    return _get_permission_from_run_id().can_update


def validate_can_delete_run():
    return _get_permission_from_run_id().can_delete


def validate_can_manage_run():
    return _get_permission_from_run_id().can_manage



from typing import Callable, List

from sqlalchemy.orm import Session

from mlflow_oidc_auth.db.models import SqlExperimentGroupRegexPermission
from mlflow_oidc_auth.entities import ExperimentGroupRegexPermission
from mlflow_oidc_auth.permissions import _validate_permission
from mlflow_oidc_auth.repository.utils import get_group, get_one_or_raise, get_user, list_user_groups, validate_regex


class ExperimentPermissionGroupRegexRepository:
    def __init__(self, session_maker):
        self._Session: Callable[[], Session] = session_maker

    def _get_experiment_group_regex_permission(self, session, regex: str, group_id: int) -> SqlExperimentGroupRegexPermission:
        """
        Get the experiment group regex permission for a given regex and group ID.
        :param session: SQLAlchemy session
        :param regex: The regex pattern.
        :param group_id: The ID of the group.
        :return: The experiment group regex permission if it exists, otherwise None.
        """
        return get_one_or_raise(
            session,
            SqlExperimentGroupRegexPermission,
            SqlExperimentGroupRegexPermission.regex == regex,
            SqlExperimentGroupRegexPermission.group_id == group_id,
            not_found_msg="Permission not found for group_id: {} and regex: {}".format(group_id, regex),
            multiple_msg="Multiple Permissions found for group_id: {} and regex: {}".format(group_id, regex),
        )

    def grant(self, group_name: str, regex: str, priority: int, permission: str) -> ExperimentGroupRegexPermission:
        _validate_permission(permission)
        validate_regex(regex)
        with self._Session() as session:
            group = get_group(session, group_name)
            perm = SqlExperimentGroupRegexPermission(regex=regex, group_id=group.id, permission=permission, priority=priority)
            session.add(perm)
            session.flush()
            return perm.to_mlflow_entity()

    def get(self, group_name: str, regex: str) -> ExperimentGroupRegexPermission:
        with self._Session() as session:
            group = get_group(session, group_name)
            row: SqlExperimentGroupRegexPermission = get_one_or_raise(
                session,
                SqlExperimentGroupRegexPermission,
                SqlExperimentGroupRegexPermission.regex == regex,
                SqlExperimentGroupRegexPermission.group_id == group.id,
                not_found_msg=f"No experiment perm for regex={regex}, group={group_name}",
                multiple_msg=f"Multiple experiment perms for regex={regex}, group={group_name}",
            )
            return row.to_mlflow_entity()

    def update(self, group_name: str, regex: str, priority: int, permission: str) -> ExperimentGroupRegexPermission:
        _validate_permission(permission)
        validate_regex(regex)
        with self._Session() as session:
            group = get_group(session, group_name)
            perm = self._get_experiment_group_regex_permission(session, regex, int(group.id))
            if perm is None:
                raise ValueError(f"No permission found for group {group_name} and regex {regex}")
            perm.permission = permission
            perm.priority = priority
            session.commit()
            return perm.to_mlflow_entity()

    def revoke(self, group_name: str, regex: str) -> None:
        validate_regex(regex)
        with self._Session() as session:
            group = get_group(session, group_name)
            perm = self._get_experiment_group_regex_permission(session, regex, int(group.id))
            if perm is None:
                raise ValueError(f"No permission found for group {group_name} and regex {regex}")
            session.delete(perm)
            session.commit()
            return None

    def list_permissions_for_group(self, group_name: str) -> List[ExperimentGroupRegexPermission]:
        with self._Session() as session:
            group = get_group(session, group_name)
            permissions = (
                session.query(SqlExperimentGroupRegexPermission)
                .filter(SqlExperimentGroupRegexPermission.group_id == group.id)
                .order_by(SqlExperimentGroupRegexPermission.priority)
                .all()
            )
            return [p.to_mlflow_entity() for p in permissions]

    def list_permissions_for_groups(self, group_names: List[str]) -> List[ExperimentGroupRegexPermission]:
        with self._Session() as session:
            group_ids = [get_group(session, group_name).id for group_name in group_names]
            permissions = (
                session.query(SqlExperimentGroupRegexPermission)
                .filter(SqlExperimentGroupRegexPermission.group_id.in_(group_ids))
                .order_by(SqlExperimentGroupRegexPermission.priority)
                .all()
            )
            return [p.to_mlflow_entity() for p in permissions]

    def list_permissions_for_group_id(self, group_id: int) -> List[ExperimentGroupRegexPermission]:
        with self._Session() as session:
            permissions = (
                session.query(SqlExperimentGroupRegexPermission)
                .filter(SqlExperimentGroupRegexPermission.group_id == group_id)
                .order_by(SqlExperimentGroupRegexPermission.priority)
                .all()
            )
            return [p.to_mlflow_entity() for p in permissions]

    def list_permissions_for_groups_ids(self, group_ids: List[int]) -> List[ExperimentGroupRegexPermission]:
        with self._Session() as session:
            permissions = (
                session.query(SqlExperimentGroupRegexPermission)
                .filter(SqlExperimentGroupRegexPermission.group_id.in_(group_ids))
                .order_by(SqlExperimentGroupRegexPermission.priority)
                .all()
            )
            return [p.to_mlflow_entity() for p in permissions]

    def list_permissions_for_user_groups(self, username: str) -> List[ExperimentGroupRegexPermission]:
        with self._Session() as session:
            user = get_user(session, username)
            user_groups = list_user_groups(session, user)
            group_ids = [group.id for group in user_groups]
            permissions = (
                session.query(SqlExperimentGroupRegexPermission)
                .filter(SqlExperimentGroupRegexPermission.group_id.in_(group_ids))
                .all()
            )
            return [p.to_mlflow_entity() for p in permissions]



from flask import Response, request
from mlflow.entities import Experiment
from mlflow.entities.model_registry import RegisteredModel
from mlflow.protos.model_registry_pb2 import CreateRegisteredModel, DeleteRegisteredModel, SearchRegisteredModels
from mlflow.protos.service_pb2 import CreateExperiment, SearchExperiments
from mlflow.server.handlers import (
    _get_model_registry_store,
    _get_request_message,
    _get_tracking_store,
    catch_mlflow_exception,
    get_endpoints,
)
from mlflow.store.entities.paged_list import PagedList
from mlflow.utils.proto_json_utils import message_to_json, parse_dict
from mlflow.utils.search_utils import SearchUtils

from mlflow_oidc_auth.permissions import MANAGE
from mlflow_oidc_auth.store import store
from mlflow_oidc_auth.utils import can_read_experiment, can_read_registered_model, get_is_admin, get_request_param, get_username


def _set_can_manage_experiment_permission(resp: Response):
    response_message = CreateExperiment.Response()  # type: ignore
    parse_dict(resp.json, response_message)
    experiment_id = response_message.experiment_id
    username = get_username()
    store.create_experiment_permission(experiment_id, username, MANAGE.name)


def _set_can_manage_registered_model_permission(resp: Response):
    response_message = CreateRegisteredModel.Response()  # type: ignore
    parse_dict(resp.json, response_message)
    name = response_message.registered_model.name
    username = get_username()
    store.create_registered_model_permission(name, username, MANAGE.name)


def _delete_can_manage_registered_model_permission(resp: Response):
    """
    Delete registered model permission when the model is deleted.

    We need to do this because the primary key of the registered model is the name,
    unlike the experiment where the primary key is experiment_id (UUID). Therefore,
    we have to delete the permission record when the model is deleted otherwise it
    conflicts with the new model registered with the same name.
    """
    # Get model name from request context because it's not available in the response
    model_name = get_request_param("name")
    store.wipe_group_model_permissions(model_name)
    store.wipe_registered_model_permissions(model_name)


def _get_after_request_handler(request_class):
    return AFTER_REQUEST_PATH_HANDLERS.get(request_class)


def _filter_search_experiments(resp: Response):
    if get_is_admin():
        return

    response_message = SearchExperiments.Response()  # type: ignore
    parse_dict(resp.json, response_message)
    # fetch permissions
    username = get_username()
    for experiment in list(response_message.experiments):
        if not can_read_experiment(experiment.experiment_id, username):
            response_message.experiments.remove(experiment)
    # re-fetch to fill max results
    request_message = _get_request_message(SearchExperiments())
    while len(response_message.experiments) < request_message.max_results and response_message.next_page_token != "":
        refetched: PagedList[Experiment] = _get_tracking_store().search_experiments(
            view_type=request_message.view_type,
            max_results=request_message.max_results,
            order_by=request_message.order_by,
            filter_string=request_message.filter,
            page_token=response_message.next_page_token,
        )
        refetched = PagedList(
            refetched[: request_message.max_results - len(response_message.experiments)],
            refetched.token if hasattr(refetched, "token") else "",
        )
        if len(refetched) == 0:
            response_message.next_page_token = ""
            break
        refetched_readable_proto = [e.to_proto() for e in refetched if can_read_experiment(e.experiment_id, username)]
        response_message.experiments.extend(refetched_readable_proto)

        # recalculate next page token
        start_offset = SearchUtils.parse_start_offset_from_page_token(response_message.next_page_token)
        final_offset = start_offset + len(refetched)
        response_message.next_page_token = SearchUtils.create_page_token(final_offset)

    resp.data = message_to_json(response_message)


def _filter_search_registered_models(resp: Response):
    if get_is_admin():
        return

    response_message = SearchRegisteredModels.Response()  # type: ignore
    parse_dict(resp.json, response_message)

    # fetch permissions
    username = get_username()
    for rm in list(response_message.registered_models):
        if not can_read_registered_model(rm.name, username):
            response_message.registered_models.remove(rm)

    # re-fetch to fill max results
    request_message = _get_request_message(SearchRegisteredModels())
    while len(response_message.registered_models) < request_message.max_results and response_message.next_page_token != "":
        refetched: PagedList[RegisteredModel] = _get_model_registry_store().search_registered_models(
            filter_string=request_message.filter,
            max_results=request_message.max_results,
            order_by=request_message.order_by,
            page_token=response_message.next_page_token,
        )
        refetched = PagedList(
            refetched[: request_message.max_results - len(response_message.registered_models)],
            refetched.token if hasattr(refetched, "token") else "",
        )
        if len(refetched) == 0:
            response_message.next_page_token = ""
            break
        refetched_readable_proto = [rm.to_proto() for rm in refetched if can_read_registered_model(rm.name, username)]
        response_message.registered_models.extend(refetched_readable_proto)

        # recalculate next page token
        start_offset = SearchUtils.parse_start_offset_from_page_token(response_message.next_page_token)
        final_offset = start_offset + len(refetched)
        response_message.next_page_token = SearchUtils.create_page_token(final_offset)

    resp.data = message_to_json(response_message)


AFTER_REQUEST_PATH_HANDLERS = {
    CreateExperiment: _set_can_manage_experiment_permission,
    CreateRegisteredModel: _set_can_manage_registered_model_permission,
    DeleteRegisteredModel: _delete_can_manage_registered_model_permission,
    SearchExperiments: _filter_search_experiments,
    SearchRegisteredModels: _filter_search_registered_models,
}

AFTER_REQUEST_HANDLERS = {
    (http_path, method): handler
    for http_path, handler, methods in get_endpoints(_get_after_request_handler)
    for method in methods
}


@catch_mlflow_exception
def after_request_hook(resp: Response):
    if 400 <= resp.status_code < 600:
        return resp

    if handler := AFTER_REQUEST_HANDLERS.get((request.path, request.method)):
        handler(resp)
    return resp



from typing import Callable, List, Optional

from mlflow.exceptions import MlflowException
from mlflow.protos.databricks_pb2 import RESOURCE_DOES_NOT_EXIST
from sqlalchemy.orm import Session

from mlflow_oidc_auth.db.models import SqlRegisteredModelGroupRegexPermission
from mlflow_oidc_auth.entities import RegisteredModelGroupRegexPermission
from mlflow_oidc_auth.permissions import _validate_permission
from mlflow_oidc_auth.repository import GroupRepository
from mlflow_oidc_auth.repository.utils import get_group


class RegisteredModelGroupRegexPermissionRepository:
    def __init__(self, session_maker):
        self._Session: Callable[[], Session] = session_maker
        self._group_repo = GroupRepository(session_maker)

    def grant(
        self, group_name: str, regex: str, permission: str, priority: int = 0, prompt: bool = False
    ) -> RegisteredModelGroupRegexPermission:
        """
        Create a new registered model group permission with regex.
        :param group_name: The name of the group.
        :param regex: The regex pattern for the registered model name.
        :param permission: The permission to be granted to the group.
        :return: The created registered model group permission.
        """
        _validate_permission(permission)
        with self._Session() as session:
            group = get_group(session, group_name)
            perm = SqlRegisteredModelGroupRegexPermission(
                regex=regex,
                priority=priority,
                group_id=group.id,
                permission=permission,
                prompt=prompt,
            )
            session.add(perm)
            session.flush()
            return perm.to_mlflow_entity()

    def get(self, regex: str, group_name: str, prompt: bool = False) -> RegisteredModelGroupRegexPermission:
        """
        Get a registered model group permission by regex and group name.
        :param regex: The regex pattern for the registered model name.
        :param group_name: The name of the group.
        :return: The registered model group permission.
        """
        with self._Session() as session:
            group = get_group(session, group_name)
            row = (
                session.query(SqlRegisteredModelGroupRegexPermission)
                .filter(
                    SqlRegisteredModelGroupRegexPermission.regex == regex,
                    SqlRegisteredModelGroupRegexPermission.group_id == group.id,
                    SqlRegisteredModelGroupRegexPermission.prompt == prompt,
                )
                .one_or_none()
            )
            if row is None:
                raise MlflowException(f"No model perm for regex={regex}, group={group_name}", RESOURCE_DOES_NOT_EXIST)
            return row.to_mlflow_entity()

    def list_permissions_for_group(self, group_name: str, prompt: bool = False) -> List[RegisteredModelGroupRegexPermission]:
        """
        List all registered model group permissions for a given group.
        :param group_name: The name of the group.
        :return: A list of registered model group permissions.
        """
        with self._Session() as session:
            group = get_group(session, group_name)
            permissions = (
                session.query(SqlRegisteredModelGroupRegexPermission)
                .filter(
                    SqlRegisteredModelGroupRegexPermission.group_id == group.id,
                    SqlRegisteredModelGroupRegexPermission.prompt == prompt,
                )
                .order_by(SqlRegisteredModelGroupRegexPermission.priority)
                .all()
            )
            return [p.to_mlflow_entity() for p in permissions]

    def list_permissions_for_groups(
        self, group_names: List[str], prompt: bool = False
    ) -> List[RegisteredModelGroupRegexPermission]:
        """
        List all registered model group permissions for a list of groups.
        :param group_names: The names of the groups.
        :return: A list of registered model group permissions.
        """
        with self._Session() as session:
            groups = [get_group(session, name) for name in group_names]
            permissions = (
                session.query(SqlRegisteredModelGroupRegexPermission)
                .filter(
                    SqlRegisteredModelGroupRegexPermission.group_id.in_([g.id for g in groups]),
                    SqlRegisteredModelGroupRegexPermission.prompt == prompt,
                )
                .order_by(SqlRegisteredModelGroupRegexPermission.priority)
                .all()
            )
            return [p.to_mlflow_entity() for p in permissions]

    def list_permissions_for_groups_ids(
        self, group_ids: List[int], prompt: bool = False
    ) -> List[RegisteredModelGroupRegexPermission]:
        """
        List all registered model group permissions for a list of groups.
        :param group_ids: The IDs of the groups.
        :return: A list of registered model group permissions.
        """
        with self._Session() as session:
            permissions = (
                session.query(SqlRegisteredModelGroupRegexPermission)
                .filter(
                    SqlRegisteredModelGroupRegexPermission.group_id.in_(group_ids),
                    SqlRegisteredModelGroupRegexPermission.prompt == prompt,
                )
                .order_by(SqlRegisteredModelGroupRegexPermission.priority)
                .all()
            )
            return [p.to_mlflow_entity() for p in permissions]

    def update(
        self, regex: str, group_name: str, permission: str, priority: int = 0, prompt: bool = False
    ) -> RegisteredModelGroupRegexPermission:
        """
        Update a registered model group permission.
        :param regex: The regex pattern for the registered model name.
        :param group_name: The name of the group.
        :param permission: The new permission to be granted to the group.
        :param priority: The new priority of the permission.
        :return: The updated registered model group permission.
        """
        _validate_permission(permission)
        with self._Session() as session:
            group = get_group(session, group_name)
            perm = (
                session.query(SqlRegisteredModelGroupRegexPermission)
                .filter(
                    SqlRegisteredModelGroupRegexPermission.regex == regex,
                    SqlRegisteredModelGroupRegexPermission.group_id == group.id,
                    SqlRegisteredModelGroupRegexPermission.prompt == prompt,
                )
                .one_or_none()
            )
            if perm is None:
                raise MlflowException(f"No perm to update for regex={regex}, group={group_name}", RESOURCE_DOES_NOT_EXIST)
            perm.permission = permission
            perm.priority = priority
            perm.prompt = prompt
            session.flush()
            return perm.to_mlflow_entity()

    def revoke(self, regex: str, group_name: str, prompt: bool = False) -> None:
        """
        Revoke a registered model group permission.
        :param regex: The regex pattern for the registered model name.
        :param group_name: The name of the group.
        """
        with self._Session() as session:
            group = get_group(session, group_name)
            perm = (
                session.query(SqlRegisteredModelGroupRegexPermission)
                .filter(
                    SqlRegisteredModelGroupRegexPermission.regex == regex,
                    SqlRegisteredModelGroupRegexPermission.group_id == group.id,
                    SqlRegisteredModelGroupRegexPermission.prompt == prompt,
                )
                .one_or_none()
            )
            if perm is None:
                raise MlflowException(f"No perm to delete for regex={regex}, group={group_name}", RESOURCE_DOES_NOT_EXIST)
            session.delete(perm)
            session.flush()



import pytest
from unittest.mock import MagicMock, patch
from mlflow_oidc_auth.repository import utils
from mlflow.exceptions import MlflowException


def test_get_one_or_raise_found():
    session = MagicMock()
    model = MagicMock()
    obj = MagicMock()
    session.query().filter().one.return_value = obj
    result = utils.get_one_or_raise(session, model, 1, not_found_msg="not found", multiple_msg="multiple")
    assert result == obj


def test_get_one_or_raise_not_found():
    session = MagicMock()
    model = MagicMock()
    session.query().filter().one.side_effect = Exception("NoResultFound")
    with patch("mlflow_oidc_auth.repository.utils.NoResultFound", Exception):
        with pytest.raises(MlflowException):
            utils.get_one_or_raise(session, model, 1, not_found_msg="not found", multiple_msg="multiple")


def test_get_one_optional_found():
    session = MagicMock()
    model = MagicMock()
    obj = MagicMock()
    session.query().filter().one_or_none.return_value = obj
    result = utils.get_one_optional(session, model, 1)
    assert result == obj


def test_get_one_optional_multiple():
    session = MagicMock()
    model = type("Model", (), {"__tablename__": "table"})
    session.query().filter().one_or_none.side_effect = Exception("MultipleResultsFound")
    with patch("mlflow_oidc_auth.repository.utils.MultipleResultsFound", Exception):
        with pytest.raises(MlflowException):
            utils.get_one_optional(session, model, 1)


def test_get_all():
    session = MagicMock()
    model = MagicMock()
    session.query().filter().all.return_value = [1, 2]
    result = utils.get_all(session, model, 1)
    assert result == [1, 2]


def test_get_user_found():
    session = MagicMock()
    user = MagicMock()
    with patch("mlflow_oidc_auth.repository.utils.get_one_or_raise", return_value=user):
        assert utils.get_user(session, "user") == user


def test_get_group_found():
    session = MagicMock()
    group = MagicMock()
    with patch("mlflow_oidc_auth.repository.utils.get_one_or_raise", return_value=group):
        assert utils.get_group(session, "group") == group


def test_list_user_groups():
    session = MagicMock()
    user = MagicMock(id=1)
    session.query().filter().all.return_value = [1, 2]
    result = utils.list_user_groups(session, user)
    assert result == [1, 2]


def test_validate_regex_valid():
    utils.validate_regex(r"^abc.*")


def test_validate_regex_empty():
    with pytest.raises(MlflowException):
        utils.validate_regex("")


def test_validate_regex_invalid():
    with pytest.raises(MlflowException):
        utils.validate_regex("[unclosed")



import os

CACHE_TYPE = "RedisCache"
CACHE_DEFAULT_TIMEOUT = os.environ.get("CACHE_DEFAULT_TIMEOUT", 300)
CACHE_KEY_PREFIX = os.environ.get("CACHE_KEY_PREFIX", "mlflow_oidc:")
CACHE_REDIS_HOST = os.environ.get("CACHE_REDIS_HOST", "localhost")
CACHE_REDIS_PORT = os.environ.get("CACHE_REDIS_PORT", 6379)
CACHE_REDIS_PASSWORD = os.environ.get("CACHE_REDIS_PASSWORD", None)
CACHE_REDIS_DB = os.environ.get("CACHE_REDIS_DB", 4)



import os

CACHE_TYPE = "FileSystemCache"
CACHE_DEFAULT_TIMEOUT = os.environ.get("CACHE_DEFAULT_TIMEOUT", 300)
CACHE_IGNORE_ERRORS = os.environ.get("CACHE_IGNORE_ERRORS", str(True)).lower() in ("true", "1", "t")
CACHE_DIR = os.environ.get("CACHE_DIR", "/tmp/flask_cache")
CACHE_THRESHOLD = os.environ.get("CACHE_THRESHOLD", 500)



import re

from flask import request
from mlflow.exceptions import MlflowException
from mlflow.protos.databricks_pb2 import RESOURCE_DOES_NOT_EXIST
from mlflow.server.handlers import _get_tracking_store

from mlflow_oidc_auth.config import config
from mlflow_oidc_auth.permissions import Permission, get_permission
from mlflow_oidc_auth.utils import effective_experiment_permission, get_experiment_id, get_request_param, get_username


def _get_permission_from_experiment_id() -> Permission:
    experiment_id = get_experiment_id()
    username = get_username()
    return effective_experiment_permission(experiment_id, username).permission


def _get_permission_from_experiment_name() -> Permission:
    experiment_name = get_request_param("experiment_name")
    store_exp = _get_tracking_store().get_experiment_by_name(experiment_name)
    if store_exp is None:
        # TODO: refactor to avoid panic during rename experiment
        raise MlflowException(
            f"Could not find experiment with name {experiment_name}",
            error_code=RESOURCE_DOES_NOT_EXIST,
        )
    username = get_username()
    return effective_experiment_permission(store_exp.experiment_id, username).permission


_EXPERIMENT_ID_PATTERN = re.compile(r"^(\d+)/")


def _get_experiment_id_from_view_args():
    # TODO: check it with get_request_param("artifact_path") to replace
    view_args = request.view_args
    if view_args is not None and (artifact_path := view_args.get("artifact_path")):
        if m := _EXPERIMENT_ID_PATTERN.match(artifact_path):
            return m.group(1)
    return None


def _get_permission_from_experiment_id_artifact_proxy() -> Permission:
    if experiment_id := _get_experiment_id_from_view_args():
        username = get_username()
        return effective_experiment_permission(experiment_id, username).permission
    return get_permission(config.DEFAULT_MLFLOW_PERMISSION)


def validate_can_read_experiment():
    return _get_permission_from_experiment_id().can_read


def validate_can_read_experiment_by_name():
    return _get_permission_from_experiment_name().can_read


def validate_can_update_experiment():
    return _get_permission_from_experiment_id().can_update


def validate_can_delete_experiment():
    return _get_permission_from_experiment_id().can_delete


def validate_can_manage_experiment():
    return _get_permission_from_experiment_id().can_manage


def validate_can_read_experiment_artifact_proxy():
    return _get_permission_from_experiment_id_artifact_proxy().can_read


def validate_can_update_experiment_artifact_proxy():
    return _get_permission_from_experiment_id_artifact_proxy().can_update


def validate_can_delete_experiment_artifact_proxy():
    return _get_permission_from_experiment_id_artifact_proxy().can_manage



import secrets

from flask import redirect, render_template, session, url_for
from mlflow.server import app

import mlflow_oidc_auth.utils as utils
from mlflow_oidc_auth.auth import get_oauth_instance
from mlflow_oidc_auth.config import config
from mlflow_oidc_auth.user import create_user, populate_groups, update_user


def login():
    state = secrets.token_urlsafe(16)
    session["oauth_state"] = state
    oauth_instance = get_oauth_instance(app)
    if oauth_instance is None or oauth_instance.oidc is None:
        app.logger.error("OAuth instance or OIDC is not properly initialized")
        return "Internal Server Error", 500
    return oauth_instance.oidc.authorize_redirect(config.OIDC_REDIRECT_URI, state=state)


def logout():
    session.clear()
    if config.AUTOMATIC_LOGIN_REDIRECT:
        return render_template(
            "auth.html",
            username=None,
            provide_display_name=config.OIDC_PROVIDER_DISPLAY_NAME,
        )
    return redirect("/")


def callback():
    """Validate the state to protect against CSRF"""

    if "oauth_state" not in session or utils.get_request_param("state") != session["oauth_state"]:
        return "Invalid state parameter", 401

    oauth_instance = get_oauth_instance(app)
    if oauth_instance is None or oauth_instance.oidc is None:
        app.logger.error("OAuth instance or OIDC is not properly initialized")
        return "Internal Server Error", 500
    token = oauth_instance.oidc.authorize_access_token()
    app.logger.debug(f"Token: {token}")
    session["user"] = token["userinfo"]

    email = token["userinfo"]["email"]
    if email is None:
        return "No email provided", 401
    display_name = token["userinfo"]["name"]
    is_admin = False
    user_groups = []

    if config.OIDC_GROUP_DETECTION_PLUGIN:
        import importlib

        user_groups = importlib.import_module(config.OIDC_GROUP_DETECTION_PLUGIN).get_user_groups(token["access_token"])
    else:
        user_groups = token["userinfo"][config.OIDC_GROUPS_ATTRIBUTE]

    app.logger.debug(f"User groups: {user_groups}")

    if config.OIDC_ADMIN_GROUP_NAME in user_groups:
        is_admin = True
    elif not any(group in user_groups for group in config.OIDC_GROUP_NAME):
        return "User is not allowed to login", 401

    create_user(username=email.lower(), display_name=display_name, is_admin=is_admin)
    populate_groups(group_names=user_groups)
    update_user(username=email.lower(), group_names=user_groups)
    session["username"] = email.lower()

    return redirect(url_for("oidc_ui"))



from flask import jsonify
from mlflow.server.handlers import catch_mlflow_exception

from mlflow_oidc_auth.store import store
from mlflow_oidc_auth.utils import check_admin_permission, get_request_param


@catch_mlflow_exception
@check_admin_permission
def create_user_experiment_regex_permission(username):
    store.create_experiment_regex_permission(
        regex=get_request_param("regex"),
        priority=int(get_request_param("priority")),
        permission=get_request_param("permission"),
        username=username,
    )
    return jsonify({"status": "success"}), 200


@catch_mlflow_exception
@check_admin_permission
def get_user_experiment_regex_permission(username):
    ep = store.list_experiment_regex_permissions(username=username)
    return jsonify([e.to_json() for e in ep]), 200


@catch_mlflow_exception
@check_admin_permission
def update_user_experiment_regex_permission(username):
    ep = store.update_experiment_regex_permission(
        username=username,
        permission=get_request_param("permission"),
        regex=get_request_param("regex"),
        priority=int(get_request_param("priority")),
    )
    return jsonify(ep.to_json()), 200


@catch_mlflow_exception
@check_admin_permission
def delete_user_experiment_regex_permission(username):
    store.delete_experiment_regex_permission(regex=get_request_param("regex"), username=username)
    return jsonify({"status": "success"}), 200


@catch_mlflow_exception
@check_admin_permission
def create_user_registered_model_regex_permission(username):
    store.create_registered_model_regex_permission(
        regex=get_request_param("regex"),
        priority=int(get_request_param("priority")),
        permission=get_request_param("permission"),
        username=username,
    )
    return jsonify({"status": "success"}), 200


@catch_mlflow_exception
@check_admin_permission
def get_user_registered_model_regex_permission(username):
    rm = store.list_registered_model_regex_permissions(username=username)
    return jsonify([r.to_json() for r in rm]), 200


@catch_mlflow_exception
@check_admin_permission
def update_user_registered_model_regex_permission(username):
    rm = store.update_registered_model_regex_permission(
        regex=get_request_param("regex"),
        priority=int(get_request_param("priority")),
        permission=get_request_param("permission"),
        username=username,
    )
    return jsonify(rm.to_json()), 200


@catch_mlflow_exception
@check_admin_permission
def delete_user_registered_model_regex_permission(username):
    store.delete_registered_model_regex_permission(regex=get_request_param("regex"), username=username)
    return jsonify({"status": "success"}), 200


@catch_mlflow_exception
@check_admin_permission
def create_user_prompt_regex_permission(username):
    store.create_prompt_regex_permission(
        regex=get_request_param("regex"),
        priority=int(get_request_param("priority")),
        permission=get_request_param("permission"),
        username=username,
    )
    return jsonify({"status": "success"}), 200


@catch_mlflow_exception
@check_admin_permission
def get_user_prompt_regex_permission(username):
    rm = store.list_prompt_regex_permissions(username=username)
    return jsonify([r.to_json() for r in rm]), 200


@catch_mlflow_exception
@check_admin_permission
def update_user_prompt_regex_permission(username):
    rm = store.update_prompt_regex_permission(
        regex=get_request_param("regex"),
        priority=int(get_request_param("priority")),
        permission=get_request_param("permission"),
        username=username,
    )
    return jsonify(rm.to_json()), 200


@catch_mlflow_exception
@check_admin_permission
def delete_user_prompt_regex_permission(username):
    store.delete_prompt_regex_permission(regex=get_request_param("regex"), username=username)
    return jsonify({"status": "success"}), 200



from unittest.mock import MagicMock, patch

import pytest
from mlflow.exceptions import MlflowException

from mlflow_oidc_auth.validators import experiment


class DummyPermission:
    def __init__(self, can_read=False, can_update=False, can_delete=False, can_manage=False):
        self.can_read = can_read
        self.can_update = can_update
        self.can_delete = can_delete
        self.can_manage = can_manage


def _patch_permission(**kwargs):
    return patch(
        "mlflow_oidc_auth.validators.experiment.effective_experiment_permission",
        return_value=MagicMock(permission=DummyPermission(**kwargs)),
    )


def test__get_permission_from_experiment_id():
    with patch("mlflow_oidc_auth.validators.experiment.get_experiment_id", return_value="123"), patch(
        "mlflow_oidc_auth.validators.experiment.get_username", return_value="alice"
    ), patch(
        "mlflow_oidc_auth.validators.experiment.effective_experiment_permission",
        return_value=MagicMock(permission=DummyPermission(can_read=True)),
    ):
        perm = experiment._get_permission_from_experiment_id()
        assert perm.can_read is True


def test__get_permission_from_experiment_name_found():
    store_exp = MagicMock()
    store_exp.experiment_id = "456"
    with patch("mlflow_oidc_auth.validators.experiment.get_request_param", return_value="expname"), patch(
        "mlflow_oidc_auth.validators.experiment._get_tracking_store"
    ) as mock_store, patch("mlflow_oidc_auth.validators.experiment.get_username", return_value="alice"), patch(
        "mlflow_oidc_auth.validators.experiment.effective_experiment_permission",
        return_value=MagicMock(permission=DummyPermission(can_update=True)),
    ):
        mock_store.return_value.get_experiment_by_name.return_value = store_exp
        perm = experiment._get_permission_from_experiment_name()
        assert perm.can_update is True


def test__get_permission_from_experiment_name_not_found():
    with patch("mlflow_oidc_auth.validators.experiment.get_request_param", return_value="expname"), patch(
        "mlflow_oidc_auth.validators.experiment._get_tracking_store"
    ) as mock_store:
        mock_store.return_value.get_experiment_by_name.return_value = None
        with pytest.raises(MlflowException):
            experiment._get_permission_from_experiment_name()


def test__get_experiment_id_from_view_args_match():
    mock_request = MagicMock()
    mock_request.view_args = {"artifact_path": "123/some/path"}
    with patch("mlflow_oidc_auth.validators.experiment.request", mock_request):
        assert experiment._get_experiment_id_from_view_args() == "123"


def test__get_experiment_id_from_view_args_no_match():
    mock_request = MagicMock()
    mock_request.view_args = {"artifact_path": "notanid/path"}
    with patch("mlflow_oidc_auth.validators.experiment.request", mock_request):
        assert experiment._get_experiment_id_from_view_args() is None


def test__get_experiment_id_from_view_args_none():
    mock_request = MagicMock()
    mock_request.view_args = None
    with patch("mlflow_oidc_auth.validators.experiment.request", mock_request):
        assert experiment._get_experiment_id_from_view_args() is None


def test__get_permission_from_experiment_id_artifact_proxy_with_id():
    with patch("mlflow_oidc_auth.validators.experiment._get_experiment_id_from_view_args", return_value="123"), patch(
        "mlflow_oidc_auth.validators.experiment.get_username", return_value="alice"
    ), patch(
        "mlflow_oidc_auth.validators.experiment.effective_experiment_permission",
        return_value=MagicMock(permission=DummyPermission(can_manage=True)),
    ):
        perm = experiment._get_permission_from_experiment_id_artifact_proxy()
        assert perm.can_manage is True


def test__get_permission_from_experiment_id_artifact_proxy_no_id():
    dummy_perm = DummyPermission(can_read=True)
    with patch("mlflow_oidc_auth.validators.experiment._get_experiment_id_from_view_args", return_value=None), patch(
        "mlflow_oidc_auth.validators.experiment.config"
    ) as mock_config, patch("mlflow_oidc_auth.validators.experiment.get_permission", return_value=dummy_perm):
        mock_config.DEFAULT_MLFLOW_PERMISSION = "default"
        perm = experiment._get_permission_from_experiment_id_artifact_proxy()
        assert perm.can_read is True


def test_validate_can_read_experiment():
    with patch("mlflow_oidc_auth.validators.experiment.get_experiment_id", return_value="123"):
        with patch("mlflow_oidc_auth.validators.experiment.get_username", return_value="alice"):
            with _patch_permission(can_read=True):
                assert experiment.validate_can_read_experiment() is True


def test_validate_can_read_experiment_by_name():
    with patch(
        "mlflow_oidc_auth.validators.experiment._get_permission_from_experiment_name",
        return_value=DummyPermission(can_read=True),
    ):
        assert experiment.validate_can_read_experiment_by_name() is True


def test_validate_can_update_experiment():
    with patch("mlflow_oidc_auth.validators.experiment.get_experiment_id", return_value="123"):
        with patch("mlflow_oidc_auth.validators.experiment.get_username", return_value="alice"):
            with _patch_permission(can_update=True):
                assert experiment.validate_can_update_experiment() is True


def test_validate_can_delete_experiment():
    with patch("mlflow_oidc_auth.validators.experiment.get_experiment_id", return_value="123"):
        with patch("mlflow_oidc_auth.validators.experiment.get_username", return_value="alice"):
            with _patch_permission(can_delete=True):
                assert experiment.validate_can_delete_experiment() is True


def test_validate_can_manage_experiment():
    with patch("mlflow_oidc_auth.validators.experiment.get_experiment_id", return_value="123"):
        with patch("mlflow_oidc_auth.validators.experiment.get_username", return_value="alice"):
            with _patch_permission(can_manage=True):
                assert experiment.validate_can_manage_experiment() is True


def test_validate_can_read_experiment_artifact_proxy():
    with patch(
        "mlflow_oidc_auth.validators.experiment._get_permission_from_experiment_id_artifact_proxy",
        return_value=DummyPermission(can_read=True),
    ):
        assert experiment.validate_can_read_experiment_artifact_proxy() is True


def test_validate_can_update_experiment_artifact_proxy():
    with patch(
        "mlflow_oidc_auth.validators.experiment._get_permission_from_experiment_id_artifact_proxy",
        return_value=DummyPermission(can_update=True),
    ):
        assert experiment.validate_can_update_experiment_artifact_proxy() is True


def test_validate_can_delete_experiment_artifact_proxy():
    with patch(
        "mlflow_oidc_auth.validators.experiment._get_permission_from_experiment_id_artifact_proxy",
        return_value=DummyPermission(can_manage=True),
    ):
        assert experiment.validate_can_delete_experiment_artifact_proxy() is True



from unittest.mock import patch
from mlflow_oidc_auth import user


class DummyUser:
    def __init__(self, username, id):
        self.username = username
        self.id = id


def test_generate_token_length_and_charset():
    token = user.generate_token()
    assert len(token) == 24
    assert all(c.isalnum() for c in token)


@patch("mlflow_oidc_auth.user.store")
def test_create_user_already_exists(mock_store):
    dummy = DummyUser("alice", 1)
    mock_store.get_user.return_value = dummy
    mock_store.update_user.return_value = None
    result = user.create_user("alice", "Alice", is_admin=True)
    assert result == (False, f"User alice (ID: 1) already exists")
    mock_store.get_user.assert_called_once_with("alice")
    mock_store.update_user.assert_called_once_with(username="alice", is_admin=True, is_service_account=False)


@patch("mlflow_oidc_auth.user.MlflowException", Exception)
@patch("mlflow_oidc_auth.user.generate_token", return_value="dummy_password")
@patch("mlflow_oidc_auth.user.store")
def test_create_user_new_user(mock_store, mock_generate_token):
    mock_store.get_user.side_effect = Exception
    dummy = DummyUser("bob", 2)
    mock_store.create_user.return_value = dummy
    result = user.create_user("bob", "Bob", is_admin=False, is_service_account=True)
    assert result == (True, f"User bob (ID: 2) successfully created")
    mock_store.create_user.assert_called_once_with(
        username="bob", password="dummy_password", display_name="Bob", is_admin=False, is_service_account=True
    )


@patch("mlflow_oidc_auth.user.store")
def test_populate_groups(mock_store):
    user.populate_groups(["g1", "g2"])
    mock_store.populate_groups.assert_called_once_with(group_names=["g1", "g2"])


@patch("mlflow_oidc_auth.user.store")
def test_update_user(mock_store):
    user.update_user("alice", ["g1", "g2"])
    mock_store.set_user_groups.assert_called_once_with("alice", ["g1", "g2"])



from unittest.mock import MagicMock, patch

from mlflow_oidc_auth.validators import run


class DummyPermission:
    def __init__(self, can_read=False, can_update=False, can_delete=False, can_manage=False):
        self.can_read = can_read
        self.can_update = can_update
        self.can_delete = can_delete
        self.can_manage = can_manage


def _patch_permission(**kwargs):
    return patch(
        "mlflow_oidc_auth.validators.run.effective_experiment_permission",
        return_value=MagicMock(permission=DummyPermission(**kwargs)),
    )


def test__get_permission_from_run_id():
    mock_run = MagicMock()
    mock_run.info.experiment_id = "exp1"
    with patch("mlflow_oidc_auth.validators.run.get_request_param", return_value="run123"), patch(
        "mlflow_oidc_auth.validators.run._get_tracking_store"
    ) as mock_store, patch("mlflow_oidc_auth.validators.run.get_username", return_value="alice"), patch(
        "mlflow_oidc_auth.validators.run.effective_experiment_permission",
        return_value=MagicMock(permission=DummyPermission(can_read=True)),
    ):
        mock_store.return_value.get_run.return_value = mock_run
        perm = run._get_permission_from_run_id()
        assert perm.can_read is True


def test_validate_can_read_run():
    mock_run = MagicMock()
    mock_run.info.experiment_id = "exp1"
    with patch("mlflow_oidc_auth.validators.run.get_request_param", return_value="run123"), patch(
        "mlflow_oidc_auth.validators.run._get_tracking_store"
    ) as mock_store, patch("mlflow_oidc_auth.validators.run.get_username", return_value="alice"):
        mock_store.return_value.get_run.return_value = mock_run
        with _patch_permission(can_read=True):
            assert run.validate_can_read_run() is True


def test_validate_can_update_run():
    mock_run = MagicMock()
    mock_run.info.experiment_id = "exp1"
    with patch("mlflow_oidc_auth.validators.run.get_request_param", return_value="run123"), patch(
        "mlflow_oidc_auth.validators.run._get_tracking_store"
    ) as mock_store, patch("mlflow_oidc_auth.validators.run.get_username", return_value="alice"):
        mock_store.return_value.get_run.return_value = mock_run
        with _patch_permission(can_update=True):
            assert run.validate_can_update_run() is True


def test_validate_can_delete_run():
    mock_run = MagicMock()
    mock_run.info.experiment_id = "exp1"
    with patch("mlflow_oidc_auth.validators.run.get_request_param", return_value="run123"), patch(
        "mlflow_oidc_auth.validators.run._get_tracking_store"
    ) as mock_store, patch("mlflow_oidc_auth.validators.run.get_username", return_value="alice"):
        mock_store.return_value.get_run.return_value = mock_run
        with _patch_permission(can_delete=True):
            assert run.validate_can_delete_run() is True


def test_validate_can_manage_run():
    mock_run = MagicMock()
    mock_run.info.experiment_id = "exp1"
    with patch("mlflow_oidc_auth.validators.run.get_request_param", return_value="run123"), patch(
        "mlflow_oidc_auth.validators.run._get_tracking_store"
    ) as mock_store, patch("mlflow_oidc_auth.validators.run.get_username", return_value="alice"):
        mock_store.return_value.get_run.return_value = mock_run
        with _patch_permission(can_manage=True):
            assert run.validate_can_manage_run() is True
